###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Drive Erase/ Sanitize Command for rdmc """

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class DriveSanitizeCommand:
    """Drive erase/sanitize command"""

    def __init__(self):
        self.ident = {
            "name": "drivesanitize",
            "usage": None,
            "description": "To sanitize a physical drive "
            'by index.\n\texample: drivesanitize "1I:1:1" --controller=1\n\n\tTo'
            " sanitize multiple drives by specifying location.\n\texample: "
            "drivesanitize 1I:1:1,1I:1:2 --controller=1 --mediatype HDD/SSD\n\texample: drivesanitize "
            "1I:1:1,1I:1:2 --controller=Slot 1 --mediatype=HDD "
            "if incorrect mediatype is specified, error will generated "
            "For iLO6, storage id need to be specified using --storageid=DE00E000 along with --controller=1\n"
            "Once drivesanitize function is performed in iLO6, It may take a while to complete.\n "
            "To check the status of Sanitization, perform to following command-\n"
            "drivesanitize 1I:1:1 --controller=1 --storageid=DE00900 --status\n"
            "Once the process in 100% complete, use the --drivereset tag to reset the drive. Example-\n"
            "drivesanitize 1I:1:1 --controller=1 --storageid=DE00900 --drivereset\n",
            "summary": "Erase/Sanitize physical drive(s)",
            "aliases": ["DriveEraseCommand"],
            "auxcommands": ["SelectCommand", "RebootCommand", "StorageControllerCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main disk inventory worker function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.drivesanitizevalidation(options)

        ilo_ver = self.rdmc.app.getiloversion()

        if ilo_ver < 6.110:
            try:
                controllers = self.auxcommands["storagecontroller"].controllers(options, single_use=True)
                if len(controllers) == 0:
                    if not options.storageid:
                        raise InvalidCommandLineError(
                            "--storageid option is mandatory. Please input storageid as well so that "
                            "controllers/volumes can be identified.\n"
                        )
                ilo_ver = 6.110
            except:
                if not options.storageid:
                    raise InvalidCommandLineError(
                        "--storageid option is mandatory. Please input storageid as well so that "
                        "controllers/volumes can be identified.\n"
                    )

        if ilo_ver >= 6.110:
            if not options.storageid:
                raise InvalidCommandLineError(
                    "--storageid option is mandatory for iLO6" " along with --controller option.\n"
                )
            self.auxcommands["select"].selectfunction("StorageController")
            content = self.rdmc.app.getprops()
            controllers = self.auxcommands["storagecontroller"].storagecontroller(options, single_use=True)
            if controllers:
                for controller in controllers:
                    if (
                        getattr(options, "controller", False) == controller
                        or controllers[controller]["Location"]["PartLocation"]["ServiceLabel"][-1]
                        == getattr(options, "controller", False)[-1]
                    ):
                        controller_physicaldrives = self.auxcommands["storagecontroller"].storagephysical_drives(
                            options,
                            options.controller,
                            options.storageid,
                            single_use=True,
                        )
        else:
            self.auxcommands["select"].selectfunction("SmartStorageConfig")
            content = self.rdmc.app.getprops()
            controllers = self.auxcommands["storagecontroller"].controllers(options, single_use=True)
            if controllers:
                for controller in controllers:
                    if int(controller) == int(options.controller):
                        controller_physicaldrives = self.auxcommands["storagecontroller"].physical_drives(
                            options, controllers[controller], single_use=True
                        )

        if not args and not options.all:
            raise InvalidCommandLineError("You must include a physical drive to sanitize.")
        elif not options.controller:
            raise InvalidCommandLineError("You must include a controller to select.")
        else:
            if len(args) > 1:
                physicaldrives = args
            elif len(args) == 1:
                physicaldrives = args[0].replace(", ", ",").split(",")
            else:
                physicaldrives = None

            controllist = []

        try:
            if ilo_ver >= 6.110:
                if options.controller.isdigit():
                    slotlocation = self.storageget_location_from_id(options.controller, options.storageid)
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip().strip("=")
                        for control in controllers.values():
                            if "Location" in control and slotcontrol.lower() == control["Location"]["PartLocation"][
                                "ServiceLabel"
                            ].lower().split("slot")[-1].lstrip().strip("="):
                                controllist.append(control)
                elif "slot" in options.controller.lower():
                    slotlocation = options.controller
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip().strip("=")
                        for control in controllers.values():
                            if "Location" in control and slotcontrol.lower() == control["Location"]["PartLocation"][
                                "ServiceLabel"
                            ].lower().split("slot")[-1].lstrip().strip("="):
                                controllist.append(control)
                if not controllist:
                    raise InvalidCommandLineError("")
            else:
                if options.controller.isdigit():
                    slotlocation = self.get_location_from_id(options.controller)
                    if slotlocation:
                        slotcontrol = slotlocation.lower().strip('"').split("slot")[-1].lstrip()
                        for control in content:
                            if slotcontrol.lower() == control["Location"].lower().split("slot")[-1].lstrip():
                                controllist.append(control)
                if not controllist:
                    raise InvalidCommandLineError("")
        except InvalidCommandLineError:
            raise InvalidCommandLineError("Selected controller not found in the current inventory " "list.")

        if ilo_ver >= 6.110:
            if options.status:
                if options.all:
                    for drive in controller_physicaldrives.values():
                        pdrive_inilo = self.convertloc(drive["PhysicalLocation"]["PartLocation"]["ServiceLabel"])
                        if len(drive["Operations"]) != 0:
                            self.rdmc.ui.printer(
                                "The drive is in %s state, %s percent complete.\n"
                                % (
                                    drive["Operations"][0]["OperationName"],
                                    drive["Operations"][0]["PercentageComplete"],
                                )
                            )
                            if int(drive["Operations"][0]["PercentageComplete"]) == 100:
                                self.rdmc.ui.printer("You can reset %s using --drivereset now.\n" % (pdrive_inilo))
                        elif len(drive["Operations"]) == 0:
                            mr_ct = drive["Status"]["State"]
                            self.rdmc.ui.printer("Sanitization is completed and updated drive status = %s\n" % (mr_ct))
                        else:
                            self.rdmc.ui.printer("Sanitization failed for drive %s.\n" % (pdrive_inilo))
                else:
                    for drive in controller_physicaldrives.values():
                        pdrive_inilo = self.convertloc(drive["PhysicalLocation"]["PartLocation"]["ServiceLabel"])
                        for userdrive in physicaldrives:
                            if userdrive == pdrive_inilo:
                                if len(drive["Operations"]) != 0:
                                    self.rdmc.ui.printer(
                                        "The drive is in %s state, %s percent complete.\n"
                                        % (
                                            drive["Operations"][0]["OperationName"],
                                            drive["Operations"][0]["PercentageComplete"],
                                        )
                                    )
                                    if int(drive["Operations"][0]["PercentageComplete"]) == 100:
                                        self.rdmc.ui.printer("You can reset %s using --drivereset now.\n" % (userdrive))
                                elif len(drive["Operations"]) == 0:
                                    mr_ct = drive["Status"]["State"]
                                    self.rdmc.ui.printer(
                                        "Sanitization is completed and updated drive status = %s\n" % (mr_ct)
                                    )

                                else:
                                    self.rdmc.ui.printer("Sanitization failed for drive %s.\n" % (userdrive))
            elif options.drivereset:
                if options.all:
                    for drive in controller_physicaldrives.values():
                        pdrive_inilo = self.convertloc(drive["PhysicalLocation"]["PartLocation"]["ServiceLabel"])
                        if "Actions" in drive:
                            if "#Drive.Reset" in drive["Actions"]:
                                path = drive["@odata.id"] + "/Actions/Drive.Reset"
                                contentsholder = {"ResetType": "ForceOn"}
                                self.rdmc.ui.printer("DriveReset path and payload: %s, %s\n" % (path, contentsholder))
                                self.rdmc.app.post_handler(path, contentsholder)
                            else:
                                if len(drive["Operations"]) != 0:
                                    self.rdmc.ui.printer(
                                        "Sanitization is in progress for drive %s. Use --status to check it's status.\n"
                                        % (pdrive_inilo)
                                    )
                                else:
                                    self.rdmc.ui.printer(
                                        "Drive sanitization is not being performed on %s\n" % (pdrive_inilo)
                                    )
                        else:
                            self.rdmc.ui.printer("Drive sanitization is not being performed on %s\n" % (pdrive_inilo))
                else:
                    for drive in controller_physicaldrives.values():
                        pdrive_inilo = self.convertloc(drive["PhysicalLocation"]["PartLocation"]["ServiceLabel"])
                        for userdrive in physicaldrives:
                            if userdrive == pdrive_inilo:
                                if "Actions" in drive:
                                    if "#Drive.Reset" in drive["Actions"]:
                                        path = drive["@odata.id"] + "/Actions/Drive.Reset"
                                        contentsholder = {"ResetType": "ForceOn"}
                                        self.rdmc.ui.printer(
                                            "DriveReset path and payload: %s, %s\n" % (path, contentsholder)
                                        )
                                        self.rdmc.app.post_handler(path, contentsholder)
                                    else:
                                        if len(drive["Operations"]) != 0:
                                            self.rdmc.ui.printer(
                                                "Santization is in progress for drive %s. Use --status to check its "
                                                "status.\n" % (pdrive_inilo)
                                            )
                                        else:
                                            self.rdmc.ui.printer(
                                                "Drive sanitization is not being performed on %s\n" % (pdrive_inilo)
                                            )
                                else:
                                    self.rdmc.ui.printer(
                                        "Drive sanitization is not being performed on %s\n" % (pdrive_inilo)
                                    )
            else:
                if self.storagesanitizedrives(physicaldrives, controller_physicaldrives, options.all):
                    if options.reboot:
                        self.auxcommands["reboot"].run("ColdBoot")
                        self.rdmc.ui.printer("Preparing for sanitization...\n")
                        self.monitorsanitization()

        else:
            if options.status is True or options.drivereset is True:
                raise InvalidCommandLineError("--status and --drivereset options are not supported on iLO 5\n")
            if self.sanitizedrives(
                controllist,
                physicaldrives,
                controller_physicaldrives,
                options.mediatype,
                options.all,
            ):
                if options.reboot:
                    self.auxcommands["reboot"].run("ColdBoot")
                    self.rdmc.ui.printer("Preparing for sanitization...\n")
                    self.monitorsanitization()
                else:
                    self.rdmc.ui.printer("Sanitization will occur on the next system reboot.\n")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def get_location_from_id(self, controller_id):
        for sel in self.rdmc.app.select("SmartStorageArrayController", path_refresh=True):
            if "Collection" not in sel.maj_type:
                controller = sel.dict
                if controller["Id"] == str(controller_id):
                    return controller["Location"]
        return None

    def storageget_location_from_id(self, controller_id, storage_id):
        for sel in self.rdmc.app.select("StorageController", path_refresh=True):
            if "Collection" not in sel.maj_type and storage_id in sel.dict["@odata.id"]:
                controller = sel.dict
                if controller["Id"] == str(controller_id):
                    return controller["Location"]["PartLocation"]["ServiceLabel"]
        return None

    def sanitizedrives(self, controllist, drivelist, controller_drives, mediatype, optall):
        """Gets drives ready for sanitization

        :param controllist: list of controllers
        :type controllist: list.
        :param drivelist: physical drives to sanitize
        :type drivelist: list.
        :param optall: flag for sanitizing all drives
        :type optall: bool.
        """
        sanitizedrivelist = []
        logicaldrivelist = []
        changes = False

        for controller in controllist:
            pdrivelist = [x["DataDrives"] for x in controller["LogicalDrives"]]

            for plist in pdrivelist:
                for drive in plist:
                    logicaldrivelist.append(drive)

            if optall:
                sanitizedrivelist = [x["Location"] for x in controller["PhysicalDrives"]]
            else:
                for erasedrive in drivelist:
                    try:
                        for idx, pdrive in enumerate(controller["PhysicalDrives"]):
                            if erasedrive == pdrive["Location"]:
                                if pdrive["Location"] in logicaldrivelist:
                                    raise InvalidCommandLineError(
                                        "Unable to"
                                        " sanitize configured drive. Remove"
                                        " any volume(s) associated "
                                        "with drive %s and try again." % pdrive["Location"]
                                    )

                                # Validate Media Type
                                if not (self.validate_mediatype(erasedrive, mediatype, controller_drives)):
                                    raise InvalidCommandLineError(
                                        "One or more of the drives given does not match the "
                                        "mediatype %s which is specified" % mediatype
                                    )
                                self.rdmc.ui.printer(
                                    "Setting physical drive %s " "for sanitization\n" % pdrive["Location"]
                                )
                                sanitizedrivelist.append(pdrive["Location"])
                                break
                    except KeyError as excp:
                        raise NoContentsFoundForOperationError(
                            "The property '%s' is missing " "or invalid." % str(excp)
                        )

            if sanitizedrivelist:
                changes = True
                if mediatype == "SSD":
                    erase_pattern_string = "SanitizeRestrictedBlockErase"
                else:
                    erase_pattern_string = "SanitizeRestrictedOverwrite"

                contentsholder = {
                    "Actions": [
                        {
                            "Action": "PhysicalDriveErase",
                            "ErasePattern": erase_pattern_string,
                            "PhysicalDriveList": sanitizedrivelist,
                        }
                    ],
                    "DataGuard": "Disabled",
                }

                # self.rdmc.ui.printer(
                #    "DriveSanitize path and payload: %s, %s\n"
                #    % (controller["@odata.id"], contentsholder)
                # )
                patch_path = controller["@odata.id"]
                if "settings" not in patch_path:
                    patch_path = patch_path + "settings/"
                controller["@odata.id"] = patch_path

                self.rdmc.app.patch_handler(controller["@odata.id"], contentsholder)

        return changes

    def convertloc(self, servicelabel):
        loc = servicelabel.split(":")
        temp_str = str(loc[1].split("=")[1] + ":" + loc[2].split("=")[1] + ":" + loc[3].split("=")[1])
        return temp_str

    def storagesanitizedrives(self, drivelist, controller_drives, optall):
        sanitizedrivelist = []
        changes = False
        plocation = ""
        if len(controller_drives) != 0:
            for plist in controller_drives.values():
                logicaldrivelist = []
                logicaldrivelist.extend(plist["Links"]["Volumes"])
                plocation = self.convertloc(plist["PhysicalLocation"]["PartLocation"]["ServiceLabel"])

                if optall:
                    if logicaldrivelist:
                        self.rdmc.ui.printer(
                            "Unable to"
                            " sanitize drive %s. Remove"
                            " any volume(s) associated "
                            "with this drive and try again.\n" % plocation
                        )
                        break
                    sanitizedrivelist.append(plocation)
                else:
                    for erasedrive in drivelist:
                        try:
                            if erasedrive == plocation:
                                if logicaldrivelist:
                                    self.rdmc.ui.printer(
                                        "Unable to"
                                        " sanitize drive %s. Remove"
                                        " any volume(s) associated "
                                        "with this drive and try again.\n" % plocation
                                    )
                                self.rdmc.ui.printer("Setting physical drive %s " "for sanitization\n" % plocation)
                                sanitizedrivelist.append(plocation)
                                break
                        except KeyError as excp:
                            raise NoContentsFoundForOperationError(
                                "The property '%s' is missing " "or invalid.\n" % str(excp)
                            )

                if sanitizedrivelist:
                    changes = True
                    path = plist["@odata.id"] + "/Actions/Drive.SecureErase"
                    contentsholder = {}

                    self.rdmc.ui.printer("DriveSanitize path and payload: %s, %s\n" % (path, contentsholder))

                    self.rdmc.app.post_handler(path, contentsholder)
                    sanitizedrivelist = []
                    self.rdmc.ui.printer("You can check the sanitization status of %s using --status\n" % (plocation))

        return changes

    def validate_mediatype(self, erasedrive, mediatype, controller_drives):
        """validates media type as HDD or SSD"""
        for idx in list(controller_drives.keys()):
            phy_drive = controller_drives[idx]
            if phy_drive["Location"] == erasedrive and phy_drive["MediaType"] == mediatype:
                return True
        return False

    def monitorsanitization(self):
        """monitors sanitization percentage"""
        # TODO: Add code to give updates on sanitization

    def drivesanitizevalidation(self, options):
        """drive sanitize validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--controller",
            dest="controller",
            help="Use this flag to select the corresponding controller " "using either the slot number or index.",
            default=None,
        )
        customparser.add_argument(
            "--storageid",
            dest="storageid",
            help="Use this flag to select the corresponding storageid in iLO6 only.",
            default=None,
        )
        customparser.add_argument(
            "--mediatype",
            dest="mediatype",
            help="""Use this flag to select the mediatype of the hard disk """,
            default=None,
            required=True,
        )
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Include this flag to perform a coldboot command "
            "function after completion of operations and monitor "
            "sanitization.",
            action="store_true",
            default=False,
        )
        customparser.add_argument(
            "--all",
            dest="all",
            help="""Use this flag to sanitize all physical drives on a """ """controller.""",
            action="store_true",
            default=False,
        )
        customparser.add_argument(
            "--drivereset",
            dest="drivereset",
            help="""Use this flag to reset physical drives on a """ """controller.""",
            action="store_true",
            default=False,
        )
        customparser.add_argument(
            "--status",
            dest="status",
            help="""Use this flag to check sanitization status of a """ """controller.""",
            action="store_true",
            default=False,
        )



"""Module containing functional tests for the device flag"""
from __future__ import absolute_import

import sys

import pytest

from rdmc_helper import ReturnCodes


@pytest.mark.skip(reason="temporarily remove display flag")
class TestDisplayFlag(object):
    """Functional Tests for the --display | -d flag"""

    @staticmethod
    def test_with_no_display_attributes(ilorest):
        """Test for --display flag with no display attributes"""
        results = ilorest.run(['showdcpmm', '--display'])
        sys.stdout.write("results.stdout: ")
        sys.stdout.write(results.stdout)
        sys.stdout.write("results.stderr: ")
        sys.stdout.write(results.stderr)
        assert results.retcode == ReturnCodes.INVALID_COMMAND_LINE_ERROR

    @staticmethod
    def test_with_invalid_display_attributes(ilorest, dl360_large):
        """Test for --display flag with invalid display attributes"""
        url, username, password = dl360_large.config
        with ilorest.session(url, username, password):
            results = ilorest.run(['showdcpmm', '--display', 'xyz,abc'])
            sys.stdout.write("results.stdout: ")
            sys.stdout.write(results.stdout)
            sys.stdout.write("results.stderr: ")
            sys.stdout.write(results.stderr)
            assert results.retcode == ReturnCodes.SUCCESS

    @staticmethod
    def test_with_valid_display_attributes(ilorest, dl360_large):
        """Test for --display flag with valid display attributes"""
        url, username, password = dl360_large.config
        with ilorest.session(url, username, password):
            results = ilorest.run(['showdcpmm', '--display', 'DeviceLocator'])
            sys.stdout.write("results.stdout: ")
            sys.stdout.write(results.stdout)
            sys.stdout.write("results.stderr: ")
            sys.stdout.write(results.stderr)
            assert results.retcode == ReturnCodes.SUCCESS

    @staticmethod
    def test_with_valid_display_attributes_json(ilorest, dl360_large):
        """
        Test for --display flag with valid display attributes
        along with the --json flag
        """
        url, username, password = dl360_large.config
        with ilorest.session(url, username, password):
            results = ilorest.run(['showdcpmm', '--display', 'DeviceLocator', '--json'])
            sys.stdout.write("results.stdout: ")
            sys.stdout.write(results.stdout)
            sys.stdout.write("results.stderr: ")
            sys.stdout.write(results.stderr)
            assert results.retcode == ReturnCodes.SUCCESS



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" List Command for RDMC """

try:
    from rdmc_helper import (
        UI,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        UI,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )


class ListCommand:
    """Constructor"""

    def __init__(self):
        self.ident = {
            "name": "list",
            "usage": None,
            "description": "Displays the current values of the "
            "properties within\n\ta selected type including"
            " reserved properties\n\texample: list\n\n\tNOTE: If "
            "you wish to not list all the reserved properties\n\t     "
            " run the get command instead",
            "summary": "Displays the current value(s) of a"
            " property(ies) within a selected type including"
            " reserved properties.",
            "aliases": ["ls"],
            "auxcommands": ["GetCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Wrapper function for main list function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        selector = options.selector.lower() if options.selector else self.rdmc.app.selector.lower()

        if "securityservice" in selector:
            self.cmdbase.login_validation(self, options)
        else:
            self.listvalidation(options)

        fvals = (None, None)

        if options.filter:
            try:
                if (str(options.filter)[0] == str(options.filter)[-1]) and str(options.filter).startswith(("'", '"')):
                    options.filter = options.filter[1:-1]

                (sel, val) = options.filter.split("=")
                fvals = (sel.strip(), val.strip())
            except:
                raise InvalidCommandLineError("Invalid filter" " parameter format [filter_attribute]=[filter_value]")
        if "StorageController." in options.selector.lower():
            options.selector = "storagecontrollercollection"
        if options.selector:
            self.rdmc.app.selector = options.selector
        elif self.rdmc.app.selector:
            options.selector = self.rdmc.app.selector
        if "securityservice" in options.selector.lower():
            url = "/redfish/v1/Managers/1/SecurityService/"
            contents = self.rdmc.app.get_handler(url, service=True, silent=True).dict
            security_contents = []

            if options and options.json and contents:
                UI().print_out_json(contents)
            elif not args:
                UI().print_out_human_readable(contents)
            else:
                attr = args[0]
                contents_lower = {k.lower(): v for k, v in contents.items()}
                security_contents.append({attr: contents_lower[attr.lower()]})
            if security_contents:
                UI().print_out_human_readable(security_contents)
        else:
            self.auxcommands["get"].getworkerfunction(args, options, filtervals=fvals, uselist=False)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def listvalidation(self, options):
        """List data validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--selector",
            dest="selector",
            help="Optionally include this flag to select a type to run"
            " the current command on. Use this flag when you wish to"
            " select a type without entering another command, or if you"
            " wish to work with a type that is different from the one"
            " you currently have selected.",
            default=None,
        )

        customparser.add_argument(
            "--filter",
            dest="filter",
            help="Optionally set a filter value for a filter attribute."
            " This uses the provided filter for the currently selected"
            " type. Note: Use this flag to narrow down your results. For"
            " example, selecting a common type might return multiple"
            " objects that are all of that type. If you want to modify"
            " the properties of only one of those objects, use the filter"
            " flag to narrow down results based on properties."
            "\t\t\t\t\t Usage: --filter [ATTRIBUTE]=[VALUE]",
            default=None,
        )
        customparser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )
        customparser.add_argument(
            "--refresh",
            dest="ref",
            action="store_true",
            help="Optionally reload the data of selected type and clear " "patches from current selection.",
            default=False,
        )



###
# Copyright 2016-2022 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Smart Nic Command for rdmc """
import os
import time

try:
    from rdmc_helper import (
        UI,
        FirmwareUpdateError,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
        UploadError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        UI,
        FirmwareUpdateError,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
        UploadError,
    )


class SmartNicCommand:
    """Smart nic command"""

    def __init__(self):
        self.ident = {
            "name": "smartnic",
            "usage": None,
            "description": "\tRun without arguments for the "
            "current list of smartnic including json format.\n\texample: "
            "smartnic/smartnic -j\n\n"
            "Show all available options\n\n\t"
            "example: smartnic --id <id> \n\t"
            "smartnic --id <id> -j\n\t"
            "smartnic --id <id> --logs\n\t"
            "smartnic --id <id> --bootprogress\n\t"
            "smartnic --id <id> --clearlog\n\t"
            "smartnic --id <id> --update_fw <fwpkg>\n\t"
            "smartnic --id <id> --reset < GracefulShutdown / ForceRestart / Nmi / GracefulRestart >\n",
            "summary": "Discovers all pensando nic installed in the " "server",
            "aliases": [],
            "auxcommands": ["SelectCommand"],
        }
        self.config_file = None
        self.fdata = None
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Smartnic command"""

        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")
        if args:
            raise InvalidCommandLineError("servernic command takes no " "arguments.")

        ilo_ver = self.rdmc.app.getiloversion()
        if ilo_ver < 5.268:
            raise IncompatibleiLOVersionError(
                "Please upgrade to iLO 5 2.68 or " "greater to ensure correct flash of this firmware."
            )

        self.smartnicvalidation(options)

        path = self.rdmc.app.typepath.defs.systempath

        info = self.gatherinfo(path)

        if not info:
            raise InvalidCommandLineError("No SmartNic available.")

        fw_version = self.get_fw_version()
        options_system = True

        if options.json and options.id is not None:
            string_id = options.id
            to_list = list(string_id.split(","))
            for id in to_list:
                if options.bootprogress:
                    if id == info["Id"]:
                        json_content = self.print_get_bootprogress(info)
                        UI().print_out_json(json_content)
                    elif id != info["Id"]:
                        self.rdmc.ui.printer("No bootprogress present for given smartnic id %s \n" % id)

                elif options.logs:
                    if id == info["Id"]:
                        json_content = self.print_get_logs(info)
                        UI().print_out_json(json_content)
                    elif id != info["Id"]:
                        self.rdmc.ui.printer("No logs present for given smartnic id %s \n" % id)

                elif options_system:
                    if id == info["Id"] and info["SystemType"] == "DPU":
                        json_content = self.build_json_out(info, fw_version)
                        UI().print_out_json(json_content)
                    elif id != info["Id"] or info["SystemType"] != "DPU":
                        self.rdmc.ui.printer("System %s:\n" % id)
                        self.rdmc.ui.printer("\tNo SmartNic present in given id %s \n" % id)

        elif options.json and options.id is None and options_system and not options.logs and not options.bootprogress:
            if info["SystemType"] == "DPU":
                json_content = self.build_json_out(info, fw_version)
                UI().print_out_json(json_content)
            if info["SystemType"] != "DPU":
                self.rdmc.ui.printer("System %s:\n" % id)
                self.rdmc.ui.printer("\tNo SmartNic present in given id %s \n" % id)

        elif (options.json and options.logs and options.id is None) or (
            options.logs and not options.json and options.id is None
        ):
            raise InvalidCommandLineError(
                "No command --logs for smartnic\n"
                "usage: smartnic --id <id> --logs \n"
                "\t  smartnic --id <id> --logs -j"
            )

        elif (options.bootprogress and options.json and options.id is None) or (
            options.bootprogress and not options.json and options.id is None
        ):
            raise InvalidCommandLineError(
                "No command --bootprogress for smartnic\n"
                "usage: smartnic --id <id> --bootprogress\n"
                "\t smartnic --id <id> --bootprogress -j"
            )

        else:
            if options.id is not None:
                string_id = options.id
                str_lst = list(string_id.split(","))
                if options.update_fw is None and options.reset is None:
                    for id in str_lst:
                        if options.bootprogress:
                            if id == info["Id"]:
                                self.get_bootprogress(info)
                            elif id != info["Id"]:
                                self.rdmc.ui.printer("No bootprogress present for given smartnic id %s\n" % id)

                        elif options.logs:
                            if id == info["Id"]:
                                self.get_logs(info)
                            elif id != info["Id"]:
                                self.rdmc.ui.printer("No logs present for given smartnic id %s \n" % id)

                        elif options_system and not options.clearlog and not options.logs:
                            if id == info["Id"] and info["SystemType"] == "DPU":
                                self.prettyprintinfo(info, fw_version)
                            elif id != info["Id"] or info["SystemType"] != "DPU":
                                self.rdmc.ui.printer("System %s:\n" % id)
                                self.rdmc.ui.printer("\tNo SmartNic present in given id %s \n" % id)

                        elif options.clearlog and options_system:
                            try:
                                for id in str_lst:
                                    if id == info["Id"]:
                                        self.clearlog(options, info, id)
                                    elif id != info["Id"]:
                                        self.rdmc.ui.printer("Given smartnic id is not present to clear the logs.")
                            except Exception as excp:
                                raise excp

                elif options.update_fw is not None:
                    try:
                        self.upload_firmware(options, info)
                    except Exception as excp:
                        # if SessionExpired:
                        # time.sleep(320)  # wait till reboot the server
                        # self.smartnicvalidation(options)
                        # self.cmdbase.login_select_validation(self, options)
                        # else:
                        raise excp

                elif options.reset is not None:
                    try:
                        for id in str_lst:
                            if id == info["Id"]:
                                self.get_resettype(options, info, id)
                            elif id != info["Id"]:
                                self.rdmc.ui.printer("No reset type present for given smartnic id %s \n" % id)

                    except Exception as excp:
                        raise excp

            else:
                if options.clearlog and options_system:
                    raise InvalidCommandLineError(
                        "No command --clearlog for smartnic\n" "usage: smartnic --id <id> --clearlog"
                    )

                if options_system and not options.clearlog:
                    if info["SystemType"] == "DPU":
                        self.prettyprintinfo(info, fw_version)
                    elif info["SystemType"] != "DPU":
                        self.rdmc.ui.printer("No SmartNic available.")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def get_fw_version(self):
        # to get pensando firmware version
        name = []
        f_version = ""
        fw_path = "/redfish/v1/UpdateService/FirmwareInventory/?$expand=."
        fw_result = self.rdmc.app.get_handler(fw_path, service=True, silent=True)
        fw_result = fw_result.dict
        members = fw_result["Members"]
        for n in members:
            name_version = n["Name"], n["Version"]
            name.append(name_version)

        for fw_name in name:
            if "Pensando" in fw_name[0]:
                f_version += fw_name[1]
        return f_version

    def clearlog(self, options, info, id):
        try:
            log_service = info["LogServices"]
            for log_id in log_service.values():
                data = self.rdmc.app.get_handler(log_id, service=True, silent=True).dict
                members = data["Members"]
                for val in members:
                    for dpu in val.values():
                        dpu_data = self.rdmc.app.get_handler(dpu, service=True, silent=True).dict
                        path = dpu_data["Actions"]["#LogService.ClearLog"]["target"]
                        action = path.split("/")[-2]
                        action = {"Action": action}
                        self.rdmc.app.post_handler(path, action)

        except Exception as excp:
            raise excp

    def get_resettype(self, options, info, id):
        self.printreboothelp(options.reset)
        time.sleep(3)

        if id == info["Id"]:
            put_path = info["@odata.id"]

        for item in info["Actions"]:
            if "Reset" in item:
                if self.rdmc.app.typepath.defs.isgen10:
                    # action = item.split("#")[-1]
                    put_path = info["Actions"][item]["target"]
                    break

        if options.reset.lower() == "gracefulshutdown":
            body = {"ResetType": "GracefulShutdown"}
        elif options.reset.lower() == "forcerestart":
            body = {"ResetType": "ForceRestart"}
        elif options.reset.lower() == "nmi":
            body = {"ResetType": "Nmi"}
        elif options.reset.lower() == "gracefulrestart":
            body = {"ResetType": "GracefulRestart"}

        self.rdmc.app.post_handler(put_path, body)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def printreboothelp(self, flag):
        if flag.upper() == "FORCERESTART":
            self.rdmc.ui.warn(
                "\nForcing a server restart. Note, the current session will be "
                "terminated.\nPlease wait for the server to boot completely before logging in "
                "again.\nRebooting the server in 3 seconds...\n"
            )
        elif flag.upper() == "NMI":
            self.rdmc.ui.warn(
                "\nA non-maskable interrupt will be issued to this server. Note, the "
                "current session will be terminated.\nIssuing interrupt in 3 seconds...\n"
            )

        elif flag.upper() == "GRACEFULSHUTDOWN":
            self.rdmc.ui.warn(
                "\nThe server will be graceful shutdown. Note, the current session will be "
                "terminated.\nPlease wait for the server to boot completely before logging in "
                "again.\nRebooting the server in 3 seconds...\n"
            )

        elif flag.upper() == "GRACEFULRESTART":
            self.rdmc.ui.warn(
                "\nGraceful server restart. Note, the current session will be "
                "terminated.\nPlease wait for the server to boot completely before logging in "
                "again.\nRebooting the server in 3 seconds...\n"
            )

    def upload_firmware(self, options, info):
        try:
            string_id = options.id
            to_list = list(string_id.split(","))
            for id in to_list:
                for info_id in info["Id"]:
                    if id == info_id:
                        target = info["@odata.id"]
                        fw_path = options.update_fw
                        results = self.rdmc.app.select(selector="UpdateService.", path_refresh=True)[0].dict
                        results = results["Actions"]["#UpdateService.SimpleUpdate"]["target"]
                        string_split = fw_path.split("/")
                        for tar in string_split:
                            if tar.endswith("tar"):
                                self.rdmc.ui.printer("Uploading firmware: %s\n" % os.path.basename(tar))

                        res = self.rdmc.app.post_handler(results, {"ImageURI": fw_path, "Targets": [target]})

                        # Taskmonitor to find the firmware status
                        task_id = res.dict["TaskMonitor"]

                        if res.status == 202:
                            status = self.wait_for_state_change(task_id, options)
                            if status:
                                self.rdmc.ui.printer(
                                    "Component " + tar + " uploaded successfully.\n"
                                    "A reboot may be required for firmware changes to take effect.\n"
                                )

                            if not status:
                                # Failed to upload the component.
                                raise UploadError("Error while processing the component.")

                        elif (res.status == 404) or (res.status == 402):
                            return ReturnCodes.FAILED_TO_UPLOAD_COMPONENT

        except (FirmwareUpdateError, UploadError) as excp:
            raise excp

    def wait_for_state_change(self, taskid, options, wait_time=4800):
        """Wait for the iLO UpdateService to a move to terminal state.
        :param options: command line options
        :type options: list.
        :param wait_time: time to wait on upload
        :type wait_time: int.
        """
        total_time = 0
        spinner = ["|", "/", "-", "\\"]
        state = ""
        self.rdmc.ui.printer("Waiting for iLO UpdateService to finish flashing the firmware \n")

        while total_time < wait_time:
            state, _ = self.get_update_service_state(taskid)
            if state == "ERROR":
                return False
            elif (
                (state == "Running")
                or (state == "New")
                or (state == "Starting")
                or (state == "Interrupted")
                or (state == "Suspended")
            ):
                # Lets try again after 8 seconds
                count = 0
                # fancy spinner
                while count <= 32:
                    self.rdmc.ui.printer("Updating: %s\r" % spinner[count % 4])
                    time.sleep(0.25)
                    count += 1
                total_time += 8
            elif state == "Completed":
                break

        if total_time >= wait_time:
            raise FirmwareUpdateError("UpdateService in " + state + " state for " + str(wait_time) + "s")

        return True

    def get_update_service_state(self, taskid):
        results = self.rdmc.app.get_handler(taskid, service=True, silent=True)
        if results.status == 202:
            return results.dict["TaskState"], results.dict

        if results and results.status == 200 and results.dict:
            output = results.dict
            err = output["error"]["@Message.ExtendedInfo"]
            for e in err:
                msgsrg = e["MessageArgs"]
            return msgsrg[0], results.dict
        else:
            return "UNKNOWN", {}

    def print_get_bootprogress(self, info):
        try:
            if info is not None:
                for id in info["Id"]:
                    content = {"Id": id}
                    content.update({"LastState": info["BootProgress"]["LastState"]})
                    content.update({"OemLastState": info["BootProgress"]["OemLastState"]})
                content_json = {"Boot Progress": content}
                return content_json
        except Exception as excp:
            raise excp

    def get_bootprogress(self, info):
        boot_output = ""
        boot_output = "------------------------------------------------\n"
        boot_output += "Boot Progress \n"
        boot_output += "------------------------------------------------\n"
        if info is not None:
            for id in info["Id"]:
                boot_output += "Id: %s\n" % id
                boot_output += "LastState: %s\n" % info["BootProgress"]["LastState"]
                boot_output += "OemLastState: %s\n" % info["BootProgress"]["OemLastState"]
            self.rdmc.ui.printer(boot_output, verbose_override=True)

    def print_get_logs(self, info):
        log_service = info["LogServices"]
        for log_id in log_service.values():
            data = self.rdmc.app.get_handler(log_id, service=True, silent=True).dict
            members = data["Members"]
            for val in members:
                for dpu in val.values():
                    dpu_data = self.rdmc.app.get_handler(dpu, service=True, silent=True).dict
                    entry = dpu_data["Entries"]
                    for dpu_entry in entry.values():
                        result = self.rdmc.app.get_handler(dpu_entry, service=True, silent=True).dict
                        result = result["Members"]
                        if not result:
                            content = {}
                        else:
                            for member in result:
                                get_res = member["@odata.id"]
                                get_result = self.rdmc.app.get_handler(get_res, service=True, silent=True).dict
                                content = {"Id": get_result["Id"]}
                                content.update({"Name": get_result["Name"]})
                                content.update({"Created": get_result["Created"]})
                                content.update({"EntryType": get_result["EntryType"]})
                                content.update({"Severity": get_result["Severity"]})
                                message = get_result["Message"]
                                message = message.split(":")
                                content.update({message[0]: message[1]})
        content_json = {"Logs View": content}
        return content_json

    def get_logs(self, info):
        log_output = ""
        log_service = info["LogServices"]
        for log_id in log_service.values():
            data = self.rdmc.app.get_handler(log_id, service=True, silent=True).dict
            members = data["Members"]
            log_output = "------------------------------------------------\n"
            log_output += "Logs View \n"
            log_output += "------------------------------------------------\n"
            for val in members:
                for dpu in val.values():
                    dpu_data = self.rdmc.app.get_handler(dpu, service=True, silent=True).dict
                    entry = dpu_data["Entries"]
                    for dpu_entry in entry.values():
                        result = self.rdmc.app.get_handler(dpu_entry, service=True, silent=True).dict
                        result = result["Members"]
                        for member in result:
                            get_res = member["@odata.id"]
                            get_result = self.rdmc.app.get_handler(get_res, service=True, silent=True).dict
                            log_output += "Id: %s\n" % get_result["Id"]
                            log_output += "Name: %s\n" % get_result["Name"]
                            log_output += "Created: %s\n" % get_result["Created"]
                            log_output += "EntryType: %s\n" % get_result["EntryType"]
                            log_output += "Severity: %s\n" % get_result["Severity"]
                            log_output += "%s\n" % get_result["Message"]
            self.rdmc.ui.printer(log_output, verbose_override=True)

    def prettyprintinfo(self, info, fw_version):
        output = ""
        output = "------------------------------------------------\n"
        output += "SmartNic Information\n"
        output += "------------------------------------------------\n"

        if info is not None:
            for id in info["Id"]:
                output += "System %s:\n" % id
                output += "\tModel: %s\n" % info["Model"]
                output += "\tManufacturer: %s\n" % info["Manufacturer"]
                output += "\tName: %s\n" % info["Name"]
                output += "\tFirmware Version: %s\n" % fw_version
                output += "\tPowerState: %s\n" % info["PowerState"]
                output += "\tSerialNumber: %s\n" % info["SerialNumber"]
                output += "\tSystemType: %s\n" % info["SystemType"]
                try:
                    output += "\tHealth: %s\n" % info["Status"]["Health"]
                except KeyError:
                    pass
                try:
                    output += "\tState: %s\n" % info["Status"]["State"]
                except KeyError:
                    pass
                try:
                    output += (
                        "\tOperating System: %s \n"
                        % info["Oem"][self.rdmc.app.typepath.defs.oemhp]["OperatingSystem"]["Kernel"]["Name"]
                    )
                except KeyError:
                    pass
                try:
                    output += (
                        "\tOS Version: %s \n"
                        % info["Oem"][self.rdmc.app.typepath.defs.oemhp]["OperatingSystem"]["Kernel"]["Version"]
                    )
                except KeyError:
                    pass
                try:
                    avail_sys = info["Oem"][self.rdmc.app.typepath.defs.oemhp]["AvailableSystemCapabilities"]
                    avail_sys = str(avail_sys).strip("[]''")
                    output += "\tAvailable SystemCapabilities: %s \n" % avail_sys
                except KeyError:
                    pass
                try:
                    enable_sys = info["Oem"][self.rdmc.app.typepath.defs.oemhp]["EnabledSystemCapabilities"]
                    enable_sys = str(enable_sys).strip("[]''")
                    output += "\tEnable SystemCapabilities: %s \n" % enable_sys
                except KeyError:
                    pass
                try:
                    integration_con = info["Oem"][self.rdmc.app.typepath.defs.oemhp]["IntegrationConfig"]
                    integration_con = str(integration_con).strip("'{} ")
                    integration_con = integration_con.replace("'", "")
                    output += "\tIntegration Config: %s \n" % integration_con
                except KeyError:
                    pass

                output += "\tUUID: %s\n" % info["UUID"]

        self.rdmc.ui.printer(output, verbose_override=True)

    def build_json_out(self, info, fw_version):
        if info is not None:
            for id in info["Id"]:
                content = {"model": info["Model"]}
                content.update({"Manufacturer": info["Manufacturer"]})
                content.update({"Name": info["Name"]})
                content.update({"Firmware Version": fw_version})
                content.update({"PowerState": info["PowerState"]})
                content.update({"SerialNumber": info["SerialNumber"]})
                content.update({"SystemType": info["SystemType"]})
                try:
                    content.update({"Health": info["Status"]["Health"]})
                except KeyError:
                    pass
                try:
                    content.update({"State": info["Status"]["State"]})
                except KeyError:
                    pass
                try:
                    content.update(
                        {
                            "Operating System": info["Oem"][self.rdmc.app.typepath.defs.oemhp]["OperatingSystem"][
                                "Kernel"
                            ]["Name"]
                        }
                    )
                except KeyError:
                    pass
                try:
                    content.update(
                        {
                            "OS Version": info["Oem"][self.rdmc.app.typepath.defs.oemhp]["OperatingSystem"]["Kernel"][
                                "Version"
                            ]
                        }
                    )
                except KeyError:
                    pass
                try:
                    content.update(
                        {
                            "Available SystemCapabilities": info["Oem"][self.rdmc.app.typepath.defs.oemhp][
                                "AvailableSystemCapabilities"
                            ]
                        }
                    )
                except KeyError:
                    pass
                try:
                    content.update(
                        {
                            "Enable SystemCapabilities": info["Oem"][self.rdmc.app.typepath.defs.oemhp][
                                "EnabledSystemCapabilities"
                            ]
                        }
                    )
                except KeyError:
                    pass
                try:
                    content.update(
                        {"Integration Config": info["Oem"][self.rdmc.app.typepath.defs.oemhp]["IntegrationConfig"]}
                    )
                except KeyError:
                    pass
                content.update({"UUID": info["UUID"]})
            system = {"System %s" % id: content}
        content_json = {"SmartNic Information": system}
        return content_json

    def gatherinfo(self, path):
        try:
            # info = {}
            results = ""
            csysresults = self.rdmc.app.select(selector="ComputerSystemCollection.")
            csysresults = csysresults[0].dict
            members = csysresults["Members"]
            for id in members:
                for mem_id in id.values():
                    if path != mem_id:
                        results = self.rdmc.app.get_handler(mem_id, service=True, silent=True).dict
            return results
        except:
            pass

    def smartnicvalidation(self, options):
        """smartnic validation function.

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )

        customparser.add_argument(
            "--id",
            dest="id",
            help="Optionally include this flag to retrive the selected smartnic .",
            default=None,
        )

        customparser.add_argument(
            "--logs",
            dest="logs",
            action="store_true",
            help="Optionally include this flag to list all the logs .",
            default=False,
        )

        customparser.add_argument(
            "--bootprogress",
            dest="bootprogress",
            action="store_true",
            help="Optionally include this flag to list the bootprogress information",
            default=False,
        )

        customparser.add_argument(
            "--update_fw",
            dest="update_fw",
            help="Include this flag to update firmware to pensando card",
            default=None,
        )
        # customparser.add_argument(
        #    "--update_os",
        #    dest="update_os",
        #    help="Include this flag to update OS to pensando card",
        #    default=None,
        # )

        customparser.add_argument(
            "--reset",
            dest="reset",
            help="Add this flag to reset the server",
            nargs="?",
            type=str,
            const="GracefulRestart",
        )

        customparser.add_argument(
            "--clearlog",
            dest="clearlog",
            action="store_true",
            help="Optionally include this flag to clear log",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" BootOrder Command for rdmc """

import ast
import copy
import fnmatch
from functools import reduce

import six

try:
    from rdmc_helper import (
        UI,
        BootOrderMissingEntriesError,
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidOrNothingChangedSettingsError,
        ReturnCodes,
    )
except:
    from ilorest.rdmc_helper import (
        UI,
        BootOrderMissingEntriesError,
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidOrNothingChangedSettingsError,
        ReturnCodes,
    )


class BootOrderCommand:
    """Changes the boot order for the server that is currently logged in"""

    def __init__(self):
        self.ident = {
            "name": "bootorder",
            "usage": None,
            "description": "Run without arguments for current boot order and one time boot options."
            "\n\texample: bootorder\n\n\tTo set the persistent boot order pick items "
            'from\n\tthe "Current Persistent Boot Order" section.\n\t'
            "example: bootorder [5,4,3,2,1] --commit\n\n\tSetting partial"
            " boot order is also supported.\n\tMissing entries are "
            "concatenated at the end.\n\texample: bootorder [5] --commit\n\n\t"
            "You can also set the boot order using partial string matching.\n\t"
            "example: bootorder NIC.*v4 HD* Generic.USB.1.1 --commit\n\n\tThis will set "
            "All v4 NICs first, followed by all hard drives,\n\tfollowed by Generic.USB.1.1. "
            "Everything not listed will be\n\tadded to the end of the boot order."
            '\n\n\tTo set one time boot entry pick items from the\n\t"'
            'Continuous and one time boot uefi options" section.\n\t'
            "example: bootorder --onetimeboot=Hdd\n\n\tTo set continuous"
            ' boot entry pick items from the\n\t"Continuous and one time '
            'boot uefi options" section.\n\texample: bootorder --'
            "continuousboot=Utilities --commit\n\n\tDisable either "
            "continuous or one time boot options.\n\texample: bootorder "
            "--disablebootflag --commit\n\n\t"
            "Changing Secure Boot Keys:\n\tTo manage secure boot keys use"
            " the --securebootkeys flag.\n\tTo delete all keys.\n\n\texample:"
            " bootorder --securebootkeys=deletepk\n\tFor all possibilities"
            " see the --securebootkeys flag \n\tin the options list.\n\n\t"
            "NOTE: pick ONETIMEBOOT and "
            'CONTINUOUS items from "Continuous\n\tand one time boot '
            'options" section. Items in this list represent\n\ta '
            '"clustered" view of the "Continuous and one time boot uefi'
            '\n\toptions" section. Example: choosing Pxe will try to Pxe'
            ' boot\n\tcapable devices in the order found in the "'
            'Continuous and one\n\ttime boot options".\n\n\t',
            "summary": "Displays and sets the current boot order.",
            "aliases": [],
            "auxcommands": [
                "GetCommand",
                "SelectCommand",
                "SetCommand",
                "RebootCommand",
            ],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main boot order worker function"""
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.bootordervalidation(options)

        if options.secureboot:
            self.secureboothelper(options.secureboot)

            if options.reboot:
                self.auxcommands["reboot"].run(options.reboot)

            return ReturnCodes.SUCCESS

        self.rdmc.app.select(selector="HpeServerBootSettings.", path_refresh=True)

        props = self.rdmc.app.getprops(skipnonsetting=False)

        for prop in props:
            bootname = prop.get("Name")

            self.rdmc.ui.printer("bootname is : {0}\n\n".format(bootname))

            if self.rdmc.app.typepath.defs.isgen10:
                if "current" in bootname.lower() or "pending" in bootname.lower():
                    try:
                        bootpath = prop.get("@odata.id")
                    except:
                        bootpath = prop.get("links")
            else:
                bootpath = "/rest/v1/systems/1/bios/boot"

        self.rdmc.ui.printer("bootpath is : {0}\n\n".format(bootpath))

        bootsources = self.rdmc.app.get_handler(bootpath, service=True, silent=True).dict["BootSources"]

        bootoverride = None
        self.auxcommands["select"].selectfunction("HpeBios.")
        try:
            bootmode = self.auxcommands["get"].getworkerfunction("BootMode", options, results=True, uselist=True)
        except:
            bootmode = dict()
            bootmode["BootMode"] = "Uefi"

        self.auxcommands["select"].selectfunction("ComputerSystem.")
        onetimebootsettings = self.auxcommands["get"].getworkerfunction(
            ["Boot/" + self.rdmc.app.typepath.defs.bootoverridetargettype],
            options,
            results=True,
            uselist=True,
        )

        bootstatus = self.auxcommands["get"].getworkerfunction(
            ["Boot/BootSourceOverrideEnabled"],
            options,
            results=True,
            uselist=True,
        )

        targetstatus = self.auxcommands["get"].getworkerfunction(
            ["Boot/BootSourceOverrideTarget"],
            options,
            results=True,
            uselist=True,
        )

        uefitargetstatus = self.auxcommands["get"].getworkerfunction(
            ["Boot/UefiTargetBootSourceOverride"],
            options,
            results=True,
            uselist=True,
        )

        currentsettings = self.rdmc.app.get_handler(self.rdmc.app.typepath.defs.systempath, service=True, silent=True)

        if bootmode and bootmode[0].get("BootMode", None) == "Uefi":
            if self.rdmc.app.typepath.defs.isgen9:
                uefionetimebootsettings = self.auxcommands["get"].getworkerfunction(
                    ["Boot/UefiTargetBootSourceOverrideSupported"],
                    options,
                    results=True,
                    uselist=True,
                )

            else:
                # Gen 10
                bootsettings = self.auxcommands["get"].getworkerfunction(
                    ["Boot"],
                    options,
                    results=True,
                    uselist=True,
                )[0]["Boot"]
                finaluefi = []
                if "UefiTargetBootSourceOverride@Redfish.AllowableValues" in bootsettings:
                    uefionetimebootsettings = bootsettings["UefiTargetBootSourceOverride@Redfish.AllowableValues"]

                    for setting in uefionetimebootsettings:
                        for source in bootsources:
                            if "UEFIDevicePath" in source and source["UEFIDevicePath"].endswith(setting):
                                finaluefi.append(source["StructuredBootString"])
                                continue

                uefionetimebootsettings = {"Boot": {"UefiTargetBootSourceOverrideSupported": finaluefi}}
        else:
            uefionetimebootsettings = None

        if options.onetimeboot is None and options.continuousboot is None and not options.disablebootflag:
            self.auxcommands["select"].selectfunction("HpeServerBootSettings.")
            bootsettings = self.auxcommands["get"].getworkerfunction(
                "PersistentBootConfigOrder", options, results=True, uselist=True
            )

            if not args:
                if not options.json:
                    self.print_out_boot_order(
                        bootsettings,
                        onetimebootsettings,
                        uefionetimebootsettings,
                        bootmode,
                        bootsources,
                        bootstatus,
                        targetstatus,
                    )
                elif options.json:
                    UI().print_out_json(bootsettings)
            elif len(args) == 1 and args[0][0] == "[":
                bootlist = args[0][1:-1].split(",")
                currentlist = bootsettings[0]["PersistentBootConfigOrder"]

                if not isinstance(currentlist, list):
                    templist = ast.literal_eval(currentlist[1:-1])
                    currentlist = [n.strip() for n in templist]

                removallist = copy.deepcopy(currentlist)

                if len(bootlist) > len(currentlist):
                    raise InvalidCommandLineError("Number of entries is greater than the current boot order length.")
                else:
                    newlist = "["

                    for value, _ in enumerate(bootlist):
                        try:
                            newlist += currentlist[int(bootlist[value]) - 1]
                        except:
                            raise InvalidCommandLineError(
                                "Invalid entry "
                                "number passed to bootorder. Please \n"
                                "       run bootorder without arguments"
                                " for possible boot \n       order numbers. "
                            )

                        removallist.remove(currentlist[int(bootlist[value]) - 1])

                        if removallist:
                            newlist += ","

                    if not removallist:
                        newlist += "]"
                    else:
                        for value, _ in enumerate(removallist):
                            newlist += removallist[value]

                            if not value == len(removallist) - 1:
                                newlist += ","

                        newlist += "]"

                    if options.biospassword:
                        newlist += " --biospassword " + options.biospassword

                    if options.reboot:
                        newlist += " --commit --reboot " + options.reboot
                    elif options.commit:
                        newlist += " --commit"

                    self.auxcommands["set"].run("PersistentBootConfigOrder=" + newlist)
            else:
                currlist = bootsettings[0]["PersistentBootConfigOrder"]
                if not isinstance(currlist, list):
                    templist = ast.literal_eval(currlist[1:-1])
                    currlist = [n.strip() for n in templist]
                remlist = copy.deepcopy(currlist)
                if len(args) > len(currlist):
                    raise InvalidCommandLineError("Number of entries is " "greater than the current boot order length.")
                newlist = []
                for arg in args:
                    argmatch = [val for val in remlist if fnmatch.fnmatch(val.lower(), arg.lower())]
                    if not argmatch and not options.ime:
                        raise InvalidCommandLineError(
                            "Invalid entry passed: "
                            "{0}. Please run bootorder to check for possible "
                            "values and reevaluate.\n".format(arg)
                        )
                    if argmatch:
                        newlist.extend(argmatch)
                        _ = [remlist.remove(val) for val in newlist if val in remlist]
                newlist.extend(remlist)
                strlist = "["
                concatlist = reduce((lambda x, y: x + "," + y), newlist)
                strlist = strlist + concatlist + "]"

                if options.biospassword:
                    strlist += " --biospassword " + options.biospassword
                if options.reboot:
                    strlist += " --commit --reboot " + options.reboot
                elif options.commit:
                    strlist += " --commit"
                self.auxcommands["set"].run("PersistentBootConfigOrder=" + strlist)
        else:
            if options.onetimeboot is not None:
                entry = options.onetimeboot

                if not bootstatus[0]["Boot"]["BootSourceOverrideEnabled"] == "Once":
                    bootoverride = " Boot/BootSourceOverrideEnabled=Once"
            elif options.continuousboot is not None:
                entry = options.continuousboot

                if not bootstatus[0]["Boot"]["BootSourceOverrideEnabled"] == "Continuous":
                    bootoverride = " Boot/BootSourceOverrideEnabled=Continuous"
            else:
                entry = "JacksBootOption"
                if not bootstatus[0]["Boot"]["BootSourceOverrideEnabled"] == "Disabled":
                    if currentsettings.dict["Boot"]["BootSourceOverrideEnabled"] == "Disabled":
                        bootoverride = "Boot/BootSourceOverrideTarget=None" " Boot/BootSourceOverrideEnabled=Disabled"
                    else:
                        bootoverride = "Boot/BootSourceOverrideTarget=None"

            newlist = ""

            if entry.lower() in (
                item.lower() for item in onetimebootsettings[0]["Boot"][self.rdmc.app.typepath.defs.bootoverridetargettype]
            ):
                if entry and isinstance(entry, six.string_types):
                    entry = entry.upper()

                entry = self.searchcasestring(
                    entry,
                    onetimebootsettings[0]["Boot"][self.rdmc.app.typepath.defs.bootoverridetargettype],
                )

                if not entry == targetstatus[0]["Boot"]["BootSourceOverrideTarget"]:
                    newlist += " Boot/BootSourceOverrideTarget=" + entry

                if bootoverride:
                    newlist += bootoverride

                if options.biospassword and newlist:
                    newlist += " --biospassword " + options.biospassword

                if options.reboot and newlist:
                    newlist += " --commit --reboot " + options.reboot
                elif options.commit and newlist:
                    newlist += " --commit"

                if newlist:
                    self.auxcommands["set"].run(newlist)
                else:
                    raise InvalidOrNothingChangedSettingsError("Entry is the current boot setting.")
            elif (
                uefionetimebootsettings
                and uefionetimebootsettings["Boot"]["UefiTargetBootSourceOverrideSupported"]
                and entry in (item for item in uefionetimebootsettings["Boot"]["UefiTargetBootSourceOverrideSupported"])
            ):
                if entry and isinstance(entry, six.string_types):
                    entry = entry.upper()

                entry = self.searchcasestring(
                    entry,
                    uefionetimebootsettings["Boot"]["UefiTargetBootSourceOverrideSupported"],
                )
                try:
                    # gen10
                    allowable_vals = next(
                        iter(
                            self.auxcommands["get"].getworkerfunction(
                                ["Boot/UefiTargetBootSourceOverride@Redfish.AllowableValues"],
                                options,
                                results=True,
                                uselist=True,
                            )
                        ),
                        {},
                    )["Boot"]["UefiTargetBootSourceOverride@Redfish.AllowableValues"]
                    for source in bootsources:
                        if source["StructuredBootString"].upper() == entry.upper():
                            for val in allowable_vals:
                                if "UEFIDevicePath" in source and source["UEFIDevicePath"].endswith(val):
                                    entry = val
                                    break
                except KeyError:
                    pass

                if not entry == uefitargetstatus["Boot"]["UefiTargetBootSourceOverride"]:
                    newlist += " Boot/UefiTargetBootSourceOverride=" + entry
                elif not targetstatus["Boot"]["BootSourceOverrideTarget"] == "UefiTarget":
                    newlist += " Boot/BootSourceOverrideTarget=UefiTarget"

                if bootoverride:
                    if self.rdmc.app.typepath.defs.isgen9 and newlist:
                        if not bootoverride.split("=")[-1] == bootstatus["Boot"]["BootSourceOverrideEnabled"]:
                            # Preemptively set UefiTargetBootSourceOverride so iLO 4 doesn't complain
                            self.rdmc.app.patch_handler(
                                self.rdmc.app.typepath.defs.systempath,
                                {"Boot": {"UefiTargetBootSourceOverride": entry}},
                                silent=True,
                                service=True,
                            )
                            self.rdmc.app.select(selector=self.rdmc.app.selector, path_refresh=True)
                            newlist = ""
                            newlist += bootoverride
                    else:
                        newlist += bootoverride

                if options.reboot and newlist:
                    newlist += " --commit --reboot " + options.reboot
                elif options.commit and newlist:
                    newlist += " --commit"

                if newlist:
                    try:
                        self.auxcommands["set"].run(newlist)
                    except InvalidOrNothingChangedSettingsError:
                        if self.rdmc.app.typepath.defs.isgen9:
                            pass
                        else:
                            raise
                else:
                    raise InvalidOrNothingChangedSettingsError("Entry is the " "current boot setting.\n")
            elif options.disablebootflag:
                if bootoverride:
                    newlist += bootoverride

                if options.reboot:
                    newlist += " --commit --reboot " + options.reboot
                elif options.commit and newlist:
                    newlist += " --commit"

                if newlist:
                    self.auxcommands["set"].run(newlist)
                else:
                    raise InvalidOrNothingChangedSettingsError("Entry is the " "current boot setting.\n")
            else:
                raise InvalidCommandLineError(
                    "Invalid entry passed for one"
                    " time boot. Please run boot \n       order without"
                    " arguments to view available options.\n"
                )

        self.cmdbase.logout_routine(self, options)

        # Return code
        return ReturnCodes.SUCCESS

    def searchcasestring(self, entry, content):
        """Helper function for retrieving correct case for value

        :param entry: entry to correlate case
        :type entry: string.
        :param content: list of items
        :type content: list.
        """
        for item in content:
            if entry.upper() == item.upper():
                return item

    def secureboothelper(self, securebootoption):
        """Helper function for secure boot function

        :param securebootoption: option passed in for secure boot
        :type securebootoption: string.
        """
        actionlist = ["defaultkeys", "deletekeys", "deletepk"]

        if not securebootoption.lower() in actionlist:
            raise InvalidCommandLineError("%s is not a valid option for " "the securebootkeys flag." % securebootoption)

        if securebootoption == actionlist[0]:
            action = "ResetAllKeysToDefault"
        elif securebootoption == actionlist[1]:
            action = "DeleteAllKeys"
        elif securebootoption == actionlist[2]:
            action = "DeletePK"

        results = self.rdmc.app.select(selector=self.rdmc.app.typepath.defs.hpsecureboot)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
            results = results.resp.dict

        try:
            for item in results["Actions"]:
                if "ResetKeys" in item:
                    path = results["Actions"][item]["target"]
                    break

            body = {"ResetKeysType": action}
            self.rdmc.app.post_handler(path, body)
        except:
            if securebootoption == actionlist[0]:
                self.auxcommands["select"].selectfunction(self.rdmc.app.typepath.defs.hpsecureboot)
                self.auxcommands["set"].run("ResetToDefaultKeys=True --commit")
            elif securebootoption == actionlist[1]:
                self.auxcommands["select"].selectfunction(self.rdmc.app.typepath.defs.hpsecureboot)
                self.auxcommands["set"].run("ResetAllKeys=True --commit")
            else:
                self.rdmc.ui.warn("DeletePK option is not available on Gen9.\n")

    def print_out_boot_order(
        self,
        content,
        onetimecontent,
        uefionetimecontent,
        bootmode,
        bootsources,
        bootstatus,
        targetstatus,
    ):
        """Convert content to human readable and print out to std.out

        :param content: current content
        :type content: string.
        :param onetimecontent: list of one time boot entries
        :type onetimecontent: list.
        :param uefionetimecontent: list of uefi one time boot entries
        :type uefionetimecontent: list.
        :param bootmode: current system boot mode
        :type bootmode: string.
        :param bootsources: current systems boot sources
        :type bootsources: list.
        """
        if content is None:
            raise BootOrderMissingEntriesError("No entries found in " "current boot order.\n\n")
        else:
            self.print_boot_helper(content[0], "\nCurrent Persistent Boot " "Order:", bootsources=bootsources)

        bootstatusval = bootstatus[0]["Boot"]["BootSourceOverrideEnabled"]
        boottoval = targetstatus[0]["Boot"]["BootSourceOverrideTarget"]
        if bootstatusval == "Continuous":
            self.rdmc.ui.printer("Current continuous boot: {0}\n\n".format(boottoval))
        elif bootstatusval == "Once":
            self.rdmc.ui.printer("Current one time boot: {0}\n\n".format(boottoval))

        if onetimecontent is None:
            raise BootOrderMissingEntriesError("No entries found for one time boot options.\n\n")
        else:
            self.print_boot_helper(onetimecontent[0]["Boot"], "Continuous and one time boot options:")

        if bootmode and any([bootmode[0].get(boot, None) == "Uefi" for boot in bootmode[0]]):
            if uefionetimecontent is None:
                self.rdmc.ui.printer("Continuous and one time boot uefi options:\n")
                self.rdmc.ui.printer(
                    "No entries found for one-time UEFI options or boot source mode " "is not set to UEFI."
                )
            else:
                if isinstance(uefionetimecontent, dict):
                    self.print_boot_helper(
                        uefionetimecontent["Boot"],
                        "Continuous and one time boot uefi options:",
                        bootsources=bootsources,
                    )
                else:
                    self.print_boot_helper(
                        uefionetimecontent[0]["Boot"],
                        "Continuous and one time boot uefi options:",
                        bootsources=bootsources,
                    )

    def print_boot_helper(self, content, outstring, indent=0, bootsources=None):
        """Print boot helper

        :param content: current content
        :type content: string.
        :param outstring: output string
        :type outstring: string.
        :param indent: indent format
        :type indent: string.
        :param bootsources: current systems boot sources
        :type bootsources: list.
        """
        for _, value in list(content.items()):
            self.rdmc.ui.printer("\t" * indent + outstring)

            if isinstance(value, list):
                count = 1

                for item in value:
                    self.rdmc.ui.printer("\n")

                    if not item:
                        item = str("null")

                    # if isinstance(item, six.string_types):
                    bootstring = False
                    try:
                        for source in bootsources:
                            if item == source["StructuredBootString"]:
                                self.rdmc.ui.printer(
                                    "\t" * indent
                                    + str(count)
                                    + ". "
                                    + str(item)
                                    + " ("
                                    + str(source["BootString"])
                                    + ")"
                                )
                                bootstring = True
                                break

                        if not bootstring:
                            self.rdmc.ui.printer("\t" * indent + str(count) + ". " + str(item))
                    except:
                        self.rdmc.ui.printer("\t" * indent + str(count) + ". " + str(item))

                    count += 1
                    # else:
                    #    self.print_boot_helper(item, indent+1)

        self.rdmc.ui.printer("\n\n")

    def bootordervalidation(self, options):
        """Boot order method validation function

        :param options: command line options
        :type options: list.
        """
        inputline = list()

        if self.rdmc.config.commit.lower() == "true":
            options.commit = True

        self.cmdbase.login_select_validation(self, options)

        if inputline:
            self.lobobj.loginfunction(inputline)

        if options.encode:
            options.biospassword = Encryption.decode_credentials(options.biospassword)
            if isinstance(options.biospassword, bytes):
                options.biospassword = options.biospassword.decode("utf-8")

    @staticmethod
    def options_argument_group(parser):
        """Additional argument

        :param parser: The parser to add the removeprivs option group to
        :type parser: ArgumentParser/OptionParser
        """

        parser.add_argument_group(
            "GLOBAL OPTION:",
            "Option(s) are available " "for all arguments within the scope of this command.",
        )

        parser.add_argument(
            "-j",
            "--json",
            dest="json",
            action="store_true",
            help="Optionally include this flag if you wish to change the"
            " displayed output to JSON format. Preserving the JSON data"
            " structure makes the information easier to parse.",
            default=False,
        )

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
        self.options_argument_group(customparser)

        customparser.add_argument(
            "--onetimeboot",
            dest="onetimeboot",
            help="Use this flag to configure a one-time boot option."
            " Using this flag will prioritize the provided boot source"
            " only on the very next time the server is booted.",
            default=None,
        )
        customparser.add_argument(
            "--continuousboot",
            dest="continuousboot",
            help="Use this flag to enable a continuous boot option. Using"
            " this flag will cause the system to boot to the selected"
            " device every time the system boots.",
            default=None,
        )
        customparser.add_argument(
            "--disablebootflag",
            dest="disablebootflag",
            action="store_true",
            help="Use this to disable either continuous or one-time boot modes.",
            default=None,
        )
        customparser.add_argument(
            "--securebootkeys",
            dest="secureboot",
            help="Use this flag to perform actions on secure boot keys."
            "Possible values include defaultkeys: resets all keys to default,"
            "deletekeys: deletes all keys, deletepk: deletes all product keys.",
            default=False,
        )
        customparser.add_argument(
            "--commit",
            dest="commit",
            action="store_true",
            help="Use this flag when you are ready to commit all pending"
            " changes. Note that some changes made in this way will be updated"
            " instantly, while others will be reflected the next time the"
            " server is started.",
            default=None,
        )
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Use this flag to perform a reboot command function after"
            " completion of operations.  For help with parameters and"
            " descriptions regarding the reboot flag, run help reboot.",
            default=None,
        )
        customparser.add_argument(
            "--ignorematcherror",
            dest="ime",
            action="store_true",
            help="Use this flag when you want to run multiple matches and "
            "not throw an error in case there are no matches found for given "
            "expression.",
            default=None,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Clear Rest API State Command for rdmc """


try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class ClearRestApiStateCommand:
    """Clear the rest api state of the server"""

    def __init__(self):
        self.ident = {
            "name": "clearrestapistate",
            "usage": None,
            "description": "Clears the persistent rest api state.\n\texample: "
            "clearrestapistate\n\n\tNote: Some types such as Bios, "
            "Iscsi, and SmartStorageConfig will be unavailable until "
            "a system reboot occurs after running this command.",
            "summary": "Clears the persistent state of the REST API. Some "
            "portions of the API may not be available until after the server reboots.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main clearrestapistate function.

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if args:
            raise InvalidCommandLineError("clearrestapistate command takes no arguments.")

        self.clearrestapistatevalidation(options)

        select = "Manager."
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("Manager. not found.")

        bodydict = results.resp.dict["Oem"][self.rdmc.app.typepath.defs.oemhp]
        try:
            for item in bodydict["Actions"]:
                if "ClearRestApiState" in item:
                    if self.rdmc.app.typepath.defs.isgen10:
                        action = item.split("#")[-1]
                    else:
                        action = "ClearRestApiState"
                    path = bodydict["Actions"][item]["target"]
                    body = {"Action": action}
                    break
        except:
            body = {"Action": "ClearRestApiState", "Target": "/Oem/Hp"}
        self.rdmc.app.post_handler(path, body)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def clearrestapistatevalidation(self, options):
        """clearrestapistate method validation function.

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)



"""Module to Test the 'applydcpmmconfig' command"""
from __future__ import absolute_import

import pytest

from rdmc_base_classes import RdmcCommandBase
from rdmc_helper import (
    InvalidCommandLineError,
    NoChangesFoundOrMadeError,
    NoContentsFoundForOperationError,
)

from ..conftest import ApplyPmemConfigCommand, RestHelpers


class MockData(object):
    """
    Mock Data for 'applydcpmmconfig' command
    """
    mock_tasks = [{
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 1",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm12/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/"
                         "MemoryChunks/?$expand=."
        },
        "TaskState": "New"
    }, {
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 2",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm10/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC2MemoryDomain/"
                         "MemoryChunks/?$expand=."
        },
        "TaskState": "Old"
    }]

    other_mock_tasks = [{
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 1",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc1dimm12/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/?$expand=."
        },
        "TaskState": "New"
    }, {
        "@odata.context": "/redfish/v1/$metadata#Task.Task",
        "Name": "Task 2",
        "Payload": {
            "HttpOperation": "POST",
            "JsonBody": "{\"AddressRangeType\": \"PMEM\","
                        "\"InterleaveSets\": "
                        "[{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm1/\"}},"
                        "{\"Memory\": "
                        "{\"@odata.id\": \"/redfish/v1/Systems/1/Memory/proc2dimm10/\"}}],"
                        "\"Oem\": {\"Hpe\": {\"MemoryChunkSizePercentage\": 0}}}",
            "TargetUri": "/redfish/v1/Systems/1/MemoryDomains/PROC2MemoryDomain/?$expand=."
        },
        "TaskState": "Old"
    }]

    mock_chunks = [
        {
            "@odata.context": "/redfish/v1/$metadata#MemoryChunks.MemoryChunks",
            "@odata.etag": "W/\"89E7F7C3\"",
            "@odata.id": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/MemoryChunks/2/",
            "@odata.type": "#MemoryChunks.v1_2_2.MemoryChunks",
            "Id": "2",
            "AddressRangeType": "PMEM",
            "Description": "Memory Chunk",
            "InterleaveSets": [
                {
                    "Memory": {
                        "@odata.id": "/redfish/v1/Systems/1/Memory/proc1dimm7/"
                    },
                    "RegionId": "3",
                    "SizeMiB": 131072
                }
            ],
            "IsMirrorEnabled": False,
            "IsSpare": False,
            "MemoryChunkSizeMiB": 131072,
            "Name": "MemoryChunk",
            "Oem": {
                "Hpe": {
                    "@odata.context": "/redfish/v1/$metadata#HpeMemoryChunksExt.HpeMemoryChunksExt",
                    "@odata.type": "#HpeMemoryChunksExt.v2_1_0.HpeMemoryChunksExt",
                    "MemoryChunkSizePercentage": 50
                }
            }
        },
        {
            "@odata.context": "/redfish/v1/$metadata#MemoryChunks.MemoryChunks",
            "@odata.etag": "W/\"89E7F7C3\"",
            "@odata.id": "/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain/MemoryChunks/1/",
            "@odata.type": "#MemoryChunks.v1_2_2.MemoryChunks",
            "Id": "1",
            "AddressRangeType": "PMEM",
            "Description": "Memory Chunk",
            "InterleaveSets": [
                {
                    "Memory": {
                        "@odata.id": "/redfish/v1/Systems/1/Memory/proc1dimm6/"
                    },
                    "RegionId": "1",
                    "SizeMiB": 131072
                }
            ],
            "IsMirrorEnabled": False,
            "IsSpare": False,
            "MemoryChunkSizeMiB": 131072,
            "Name": "MemoryChunk",
            "Oem": {
                "Hpe": {
                    "@odata.context": "/redfish/v1/$metadata#HpeMemoryChunksExt.HpeMemoryChunksExt",
                    "@odata.type": "#HpeMemoryChunksExt.v2_1_0.HpeMemoryChunksExt",
                    "MemoryChunkSizePercentage": 50
                }
            }
        }
    ]

    mock_interleavable_set = [
        {
            "MemorySet": [
                {
                    "@odata.id": "/redfish/v1/Systems/1/Memory/proc2dimm6/"
                },
                {
                    "@odata.id": "/redfish/v1/Systems/1/Memory/proc2dimm7/"
                }
            ]
        },
        {
            "MemorySet": [
                {
                    "@odata.id": "/redfish/v1/Systems/1/Memory/proc2dimm6/"
                }
            ]
        },
        {
            "MemorySet": [
                {
                    "@odata.id": "/redfish/v1/Systems/1/Memory/proc2dimm7/"
                }
            ]
        }
    ]

    expect_post_bodies_app_direct_interleaved = [
        {
            'AddressRangeType': 'PMEM',
            'Oem': {
                'Hpe': {
                    'MemoryChunkSizePercentage': 100
                }
            },
            'InterleaveSets': [
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm6/'
                    }
                },
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm7/'
                    }
                }
            ]
        }
    ]

    expected_post_bodies_app_direct_not_interleaved = [
        {
            'AddressRangeType': 'PMEM',
            'Oem': {
                'Hpe': {
                    'MemoryChunkSizePercentage': 100
                }
            },
            'InterleaveSets': [
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm6/'
                    }
                }
            ]
        },
        {
            'AddressRangeType': 'PMEM',
            'Oem': {
                'Hpe': {
                    'MemoryChunkSizePercentage': 100
                }
            },
            'InterleaveSets': [
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm7/'
                    }
                }
            ]
        }
    ]

    expected_post_bodies_memory_mode = [
        {
            'AddressRangeType': 'PMEM',
            'Oem': {
                'Hpe': {
                    'MemoryChunkSizePercentage': 0
                }
            },
            'InterleaveSets': [
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm6/'
                    }
                },
                {
                    'Memory': {
                        '@odata.id': '/redfish/v1/Systems/1/Memory/proc2dimm7/'
                    }
                }
            ]
        }

    ]

    def retrieve_memory_domains(self):
        """
        Function to retrive memory domains
        """
        memory_domains = [{
            'Name': 'PROC1 Memory Domain',
            '@odata.id': '/redfish/v1/Systems/1/MemoryDomains/PROC1MemoryDomain',
            '@odata.context': '/redfish/v1/$metadata#MemoryDomain.MemoryDomain',
            'Id': 'PROC1MemoryDomain',
            "InterleavableMemorySets":[{
                "MemorySet":[
                    {
                        "@odata.id":"/redfish/v1/Systems/1/Memory/proc2dimm6/"
                    },
                    {
                        "@odata.id":"/redfish/v1/Systems/1/Memory/proc2dimm7/"
                    }
                ]
            }, {
                "MemorySet":[
                    {
                        "@odata.id":"/redfish/v1/Systems/1/Memory/proc2dimm6/"
                    }
                ]
            }, {
                "MemorySet":[
                    {
                        "@odata.id":"/redfish/v1/Systems/1/Memory/proc2dimm7/"
                    }
                ]
            }],
            "MemoryChunks":{
                "@odata.id":"/redfish/v1/Systems/1/MemoryDomains/PROC2MemoryDomain/MemoryChunks"}
        }]
        return [], memory_domains, []

class TestApplyPmemConfigCommand(object):
    """
    Class for unit tests of 'applydcpmmconfig' command
    """

    @staticmethod
    @pytest.mark.parametrize('customparser, expected_result', [
        ('', None)
    ])
    def test_define_arguments(customparser, expected_result):
        """
        Branch Test for 'define_arguments()' when parser is not customparser
        """
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(
            RdmcCommandBase)
        result = pmem_command.define_arguments(customparser)
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize('memory_chunk_tasks, memory_chunks', [
        ([MockData.mock_tasks[0]], list()),
        (list(), MockData.mock_chunks),
        ([MockData.mock_tasks[1]], MockData.mock_chunks),
        ([MockData.other_mock_tasks[0]], MockData.mock_chunks)
    ])
    def test_warn_existing_chunks_and_tasks_fail(memory_chunk_tasks, memory_chunks):
        """
        Tests for 'delete_existing_chunks_and_tasks' with invalid data
        """
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        with pytest.raises(NoChangesFoundOrMadeError):
            pmem_command.warn_existing_chunks_and_tasks(memory_chunk_tasks, memory_chunks)

    @staticmethod
    @pytest.mark.parametrize('memory_chunk_tasks, memory_chunks, expected_result', [
        (list(), list(), None),
        (None, list(), None)
    ])
    def test_warn_existing_chunks_and_tasks_pass(memory_chunk_tasks,
                                                 memory_chunks, expected_result):
        """
        Tests for 'delete_existing_chunks_and_tasks' with valid data
        """
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        result = pmem_command.warn_existing_chunks_and_tasks(memory_chunk_tasks, memory_chunks)
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize('memory_chunk_tasks, memory_chunks, response, expected_result', [
        ([MockData.mock_tasks[0]], list(), 200, None),
        ([MockData.mock_tasks[0]], MockData.mock_chunks, 202, None),
        (None, None, None, None),
        (list(), list(), None, None)
    ])
    def test_delete_existing_chunks_and_tasks_pass(memory_chunk_tasks, memory_chunks,
                                                   response, expected_result, monkeypatch):
        """
        Tests for 'delete_existing_chunks_and_tasks' with valid data
        """
        def get_response(*argv):
            """
            Returns response codes
            """
            #pylint: disable=unused-argument
            return response
        monkeypatch.setattr(RestHelpers.RestHelpers, "delete_resource", get_response)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        result = pmem_command.delete_existing_chunks_and_tasks(memory_chunk_tasks, memory_chunks)
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize('memory_chunk_tasks, memory_chunks', [
        ([MockData.mock_tasks[0]], MockData.mock_chunks),
        (None, MockData.mock_chunks),
    ])
    def test_delete_existing_chunks_and_tasks_fail(memory_chunk_tasks, memory_chunks, monkeypatch):
        """
        Tests for 'delete_existing_chunks_and_tasks' with invalid data
        """
        def get_response(*argv):
            """
            Returns none
            """
            #pylint: disable=unused-argument
            return None
        monkeypatch.setattr(RestHelpers.RestHelpers, "delete_resource", get_response)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        with pytest.raises(NoChangesFoundOrMadeError):
            pmem_command.delete_existing_chunks_and_tasks(memory_chunk_tasks, memory_chunks)


class MockOptions(object):
    """
    Class containing mock options for validation tests
    """
    list = False
    config = ""
    force = False

    @staticmethod
    def set_options(list, config, force):
        """
        Setter function for options
        """
        MockOptions.list = list
        MockOptions.config = config
        MockOptions.force = force


class TestValidation(object):
    """
    Class containing test for validation based on mock options
    """
    @staticmethod
    @pytest.mark.parametrize("list, config, force, expected_result", [
        (True, False, False, None),
        (False, "memorymode", False, None),
        (False, "appdirectnotinterleaved", True, None),
        (False, "appdirectinterleaved", True, None),
        (False, "AppdirecTinterLeaveD", False, None),
        (False, "APPDIRECTNOTINTERLEAVED", False, None),
    ])
    def test_validate_options_pass(list, config, force, expected_result):
        """
        Tests for 'validate_options' with valid options
        """
        MockOptions.set_options(list, config, force)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        result = pmem_command.validate_options(MockOptions)
        assert result == expected_result

    @staticmethod
    @pytest.mark.parametrize("list_flag, config, force", [
        (False, "", False),
        (True, "AppDirectInterleaved", False),
        (False, "qwerty", True),
        (True, "MemoryMode", True),
        (False, None, True),
        (True, None, True)
    ])
    def test_validate_options_fail(list_flag, config, force):
        """
        Tests for 'validate_options' with invalid options
        """
        MockOptions.set_options(list_flag, config, force)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        with pytest.raises(InvalidCommandLineError):
            pmem_command.validate_options(MockOptions)

    @staticmethod
    @pytest.mark.parametrize("list_flag, config, force", [
        (False, "False", False),
        (False, None, True),
        (True, "args", True)
    ])
    def test_validate_args(list_flag, config, force):
        """
        Tests for 'validate_args'
        """
        MockOptions.set_options(list_flag, config, force)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        with pytest.raises(InvalidCommandLineError):
            pmem_command.validate_args(MockOptions)

    @staticmethod
    @pytest.mark.parametrize('mock_config_data, mock_interleavable_set, expected_result', [
        (ApplyPmemConfigCommand.ApplyPmemConfigCommand.config_ids[0],
         MockData.mock_interleavable_set,
         MockData.expected_post_bodies_memory_mode),
        (ApplyPmemConfigCommand.ApplyPmemConfigCommand.config_ids[1],
         MockData.mock_interleavable_set,
         MockData.expect_post_bodies_app_direct_interleaved),
        (ApplyPmemConfigCommand.ApplyPmemConfigCommand.config_ids[2],
         MockData.mock_interleavable_set,
         MockData.expected_post_bodies_app_direct_not_interleaved)
    ])

    def test_get_post_data(mock_config_data, mock_interleavable_set, expected_result):
        """
        Test for get_post_data with MemoryMode, AppDirectInterleaved and AppDirectNotInterleaved
        """
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(
            RdmcCommandBase)
        result = pmem_command.get_post_data(mock_config_data, mock_interleavable_set)
        assert result == expected_result

    @staticmethod
    def test_apply_predefined_err(monkeypatch):
        """
        Test for apply_predefined_config with empty memory domains list
        """
        def retrive_empty_mem_domains(*argv):
            """
            returns empty list
            """
            #pylint: disable=unused-argument
            return [], [], []

        MockOptions.set_options(False, "memorymode", True)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        monkeypatch.setattr(RestHelpers.RestHelpers, "retrieve_task_members_and_mem_domains",
                            retrive_empty_mem_domains)
        with pytest.raises(NoContentsFoundForOperationError):
            pmem_command.apply_predefined_config(MockOptions)

    def test_apply_predefined_err3(self, monkeypatch):
        """
        Test for apply_predefined_config with invalid post request
        """
        def get_failure_resp(*argv):
            """
            returns None
            """
            #pylint: disable=unused-argument
            return None
        MockOptions.set_options(False, "appdirectinterleaved", True)
        pmem_command = ApplyPmemConfigCommand.ApplyPmemConfigCommand(RdmcCommandBase)
        monkeypatch.setattr(RestHelpers.RestHelpers, "retrieve_task_members_and_mem_domains",
                            MockData().retrieve_memory_domains)
        monkeypatch.setattr(RestHelpers.RestHelpers, "post_resource", get_failure_resp)
        with pytest.raises(NoChangesFoundOrMadeError):
            pmem_command.apply_predefined_config(MockOptions)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Commit Command for RDMC """

from redfish.ris.rmc_helper import NothingSelectedError

try:
    from rdmc_helper import (
        FailureDuringCommitError,
        InvalidCommandLineErrorOPTS,
        NoChangesFoundOrMadeError,
        NoCurrentSessionEstablished,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        FailureDuringCommitError,
        InvalidCommandLineErrorOPTS,
        NoChangesFoundOrMadeError,
        NoCurrentSessionEstablished,
        ReturnCodes,
    )


class CommitCommand:
    """Constructor"""

    def __init__(self):
        self.ident = {
            "name": "commit",
            "usage": None,
            "description": "commit [OPTIONS]\n\n\tRun to apply all changes made during"
            " the current session\n\texample: commit",
            "summary": "Applies all the changes made during the current session.",
            "aliases": [],
            "auxcommands": ["LogoutCommand", "RebootCommand"],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def commitfunction(self, options=None):
        """Main commit worker function

        :param options: command line options
        :type options: list.
        """
        self.commitvalidation()

        self.rdmc.ui.printer("Committing changes...\n")

        if options:
            if options.biospassword:
                self.rdmc.app.current_client.bios_password = options.biospassword
        try:
            failure = False
            commit_opp = self.rdmc.app.commit()
            for path in commit_opp:
                if self.rdmc.opts.verbose:
                    self.rdmc.ui.printer("Changes are being made to path: %s\n" % path)
                if next(commit_opp):
                    failure = True
        except NothingSelectedError:
            raise NoChangesFoundOrMadeError("No changes found or made during commit operation.")
        else:
            if failure:
                raise FailureDuringCommitError(
                    "One or more types failed to commit. Run the "
                    "status command to see uncommitted data. "
                    "if you wish to discard failed changes refresh the "
                    "type using select with the --refresh flag."
                )

        if options.reboot:
            self.auxcommands["reboot"].run(options.reboot)
            self.auxcommands["logout"].run("")

    def run(self, line, help_disp=False):
        """Wrapper function for commit main function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.commitfunction(options)

        # Return code
        return ReturnCodes.SUCCESS

    def commitvalidation(self):
        """Commit method validation function"""

        try:
            _ = self.rdmc.app.current_client
        except:
            raise NoCurrentSessionEstablished("Please login and make setting" " changes before using commit command.")

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Use this flag to perform a reboot command function after"
            " completion of operations.  For help with parameters and"
            " descriptions regarding the reboot flag, run help reboot.",
            default=None,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Firmware Update Command for rdmc """

import time

try:
    from rdmc_helper import (
        IloLicenseError,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
        TimeOutError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        IloLicenseError,
        IncompatibleiLOVersionError,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
        TimeOutError,
    )


class FirmwareIntegrityCheckCommand:
    """Reboot server that is currently logged in"""

    def __init__(self):
        self.ident = {
            "name": "fwintegritycheck",
            "usage": None,
            "description": "Perform a firmware "
            "integrity check on the current logged in server.\n\t"
            "example: fwintegritycheck\n\n\tPerform a firmware integrity check and "
            "return results of the check.\n\texmaple: fwintegritycheck --results",
            "summary": "Perform a firmware integrity check on the currently logged in server.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main firmware update worker function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        if args:
            raise InvalidCommandLineError("fwintegritycheck command takes no arguments")

        self.firmwareintegritycheckvalidation(options)
        if self.rdmc.app.typepath.defs.isgen9:
            raise IncompatibleiLOVersionError("fwintegritycheck command is " "only available on iLO 5.")

        licenseres = self.rdmc.app.select(selector="HpeiLOLicense.")
        try:
            licenseres = licenseres[0]
        except:
            pass
        if not licenseres.dict["LicenseFeatures"]["FWScan"]:
            raise IloLicenseError("This command is not available with this iLO license.\n")

        select = self.rdmc.app.typepath.defs.hpilofirmwareupdatetype
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        bodydict = results.resp.dict

        path = bodydict["Oem"]["Hpe"]["Actions"]["#HpeiLOUpdateServiceExt.StartFirmwareIntegrityCheck"]["target"]

        self.rdmc.app.post_handler(path, {})

        if options.results:
            results_string = "Awaiting results of firmware integrity check..."
            self.rdmc.ui.printer(results_string)
            polling = 50
            found = False
            while polling > 0:
                if not polling % 5:
                    self.rdmc.ui.printer(".")
                get_results = self.rdmc.app.get_handler(bodydict["@odata.id"], service=True, silent=True)
                if get_results:
                    curr_time = time.strptime(bodydict["Oem"]["Hpe"]["CurrentTime"], "%Y-%m-%dT%H:%M:%SZ")
                    scan_time = time.strptime(
                        get_results.dict["Oem"]["Hpe"]["FirmwareIntegrity"]["LastScanTime"],
                        "%Y-%m-%dT%H:%M:%SZ",
                    )

                    if scan_time > curr_time:
                        self.rdmc.ui.printer(
                            "\nScan Result: %s\n"
                            % get_results.dict["Oem"]["Hpe"]["FirmwareIntegrity"]["LastScanResult"]
                        )
                        found = True
                        break

                    polling -= 1
                    time.sleep(1)
            if not found:
                self.rdmc.ui.error("\nPolling timed out before scan completed.\n")
                TimeOutError("")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def firmwareintegritycheckvalidation(self, options):
        """Firmware update method validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--results",
            dest="results",
            help="Optionally include this flag to show results of firmware integrity check.",
            default=False,
            action="store_true",
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" RawGet Command for rdmc """

import json

import redfish

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ResourceNotReadyError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ResourceNotReadyError,
        ReturnCodes,
    )


class RawGetCommand:
    """Raw form of the get command"""

    def __init__(self):
        self.ident = {
            "name": "rawget",
            "usage": None,
            "description": "Run to to retrieve data from "
            'the passed in path.\n\tExample: rawget "/redfish/v1/'
            'systems/(system ID)"',
            "summary": "Raw form of the GET command.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main raw get worker function

        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        headers = {}

        if hasattr(options, "sessionid") and options.sessionid:
            _ = self.sessionvalidation(options)
        else:
            self.getvalidation(options)

        if options.path.endswith("?=."):
            path = options.path
            strip = path[:-3]
            options.path = strip + "?$expand=."

        if options.path.startswith('"') and options.path.endswith('"'):
            options.path = options.path[1:-1]

        if options.expand:
            options.path = options.path + "?$expand=."

        if options.headers:
            extraheaders = options.headers.split(",")
            for item in extraheaders:
                header = item.split(":")

                try:
                    headers[header[0]] = header[1]
                except:
                    InvalidCommandLineError("Invalid format for --headers " "option.")

        returnresponse = False
        if options.response or options.getheaders:
            returnresponse = True

        extra_path = None
        if "#" in options.path:
            path_list = options.path.split("#")
            options.path = path_list[0]
            extra_path = path_list[1]
            if "/" in extra_path:
                extra_path_list = extra_path.split("/")
                extra_path_list = list(filter(None, extra_path_list))

        results = self.rdmc.app.get_handler(
            options.path,
            sessionid=options.sessionid,
            headers=headers,
            silent=options.silent,
            service=options.service,
            username=options.user,
            password=options.password,
            base_url=options.url,
        )
        result = None
        if results.dict:
            if extra_path:
                result = results.dict
                for p in extra_path_list:
                    if p.isdigit():
                        p = int(p)
                    result = result[p]
            else:
                result = results.dict

        if results and results.status == 200 and options.binfile:
            output = results.read
            filehndl = open(options.binfile[0], "wb")
            filehndl.write(output)
            filehndl.close()
        elif results and returnresponse:
            if options.getheaders:
                self.rdmc.ui.printer(json.dumps(dict(results.getheaders())) + "\n")
            if options.response:
                self.rdmc.ui.printer(results.read + "\n")
        elif results and results.status == 200:
            if results.dict:
                if options.filename:
                    output = json.dumps(
                        result,
                        indent=2,
                        cls=redfish.ris.JSONEncoder,
                        sort_keys=True,
                    )

                    filehndl = open(options.filename[0], "w")
                    filehndl.write(output)
                    filehndl.close()

                    self.rdmc.ui.printer("Results written out to '%s'.\n" % options.filename[0])
                else:
                    if not result:
                        result = results.dict
                    if options.service:
                        self.rdmc.ui.printer("%s\n" % result)
                    else:
                        self.rdmc.ui.print_out_json(result)
        else:
            json_payload = json.loads(results._http_response.data)
            try:
                message_id = json_payload["error"]["@Message.ExtendedInfo"][0]["MessageId"]
                self.rdmc.ui.error("%s" % message_id)
                if "ResourceNotReadyRetry" in message_id:
                    raise ResourceNotReadyError("Resources are not ready in iLO, Please wait for some time and retry")
            except:
                self.rdmc.ui.error("An invalid or incomplete response was received: %s\n" % json_payload)
            return ReturnCodes.NO_CONTENTS_FOUND_FOR_OPERATION

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def getvalidation(self, options):
        """Raw get validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options, skipbuild=True)

    def sessionvalidation(self, options):
        """Raw patch session validation function

        :param options: command line options
        :type options: list.
        """

        url = None
        if options.user or options.password or options.url:
            if options.url:
                url = options.url
        else:
            if self.rdmc.app.redfishinst and self.rdmc.app.redfishinst.base_url:
                url = self.rdmc.app.redfishinst.base_url
        if url and "blobstore://" not in url and "https://" not in url:
            url = "https://" + url

        return url

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "path",
            help="Uri on iLO",
        )
        customparser.add_argument(
            "--response",
            dest="response",
            action="store_true",
            help="Use this flag to return the iLO response body.",
            default=False,
        )
        customparser.add_argument(
            "--getheaders",
            dest="getheaders",
            action="store_true",
            help="Use this flag to return the iLO response headers.",
            default=False,
        )
        customparser.add_argument(
            "--headers",
            dest="headers",
            help="Use this flag to add extra headers to the request." " example: --headers=HEADER:VALUE,HEADER:VALUE",
            default=None,
        )
        customparser.add_argument(
            "--silent",
            dest="silent",
            action="store_true",
            help="""Use this flag to silence responses""",
            default=False,
        )
        customparser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="""Write results to the specified file.""",
            action="append",
            default=None,
        )
        customparser.add_argument(
            "-b",
            "--writebin",
            dest="binfile",
            help="""Write the results to the specified file in binary.""",
            action="append",
            default=None,
        )
        customparser.add_argument(
            "--service",
            dest="service",
            action="store_true",
            help="""Use this flag to enable service mode and increase the function speed""",
            default=False,
        )
        customparser.add_argument(
            "--expand",
            dest="expand",
            action="store_true",
            help="""Use this flag to expand the path specified using the """ """expand notation '?$expand=.'""",
            default=False,
        )



# ##
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# ##

# -*- coding: utf-8 -*-
""" Fwpkg Command for rdmc """

import os
import json
import sys
from random import randint
import shutil
import zipfile
import tempfile
from io import open

import ctypes
from ctypes import c_char_p, c_int, c_bool

from redfish.hpilo.risblobstore2 import BlobStore2

try:
    from rdmc_helper import (
        LOGGER,
        IncompatibleiLOVersionError,
        ReturnCodes,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        UploadError,
        TaskQueueError,
        FirmwareUpdateError,
        UploadPowerOffError,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        LOGGER,
        IncompatibleiLOVersionError,
        ReturnCodes,
        InvalidCommandLineErrorOPTS,
        InvalidFileInputError,
        UploadError,
        TaskQueueError,
        FirmwareUpdateError,
    )


class FwpkgCommand:
    """Fwpkg command class"""

    def __init__(self):
        self.ident = {
            "name": "flashfwpkg",
            "usage": None,
            "description": "Run to upload and flash "
            "components from fwpkg files.\n\n\tUpload component and flashes it or sets a task"
            "queue to flash.\n\texample: flashfwpkg component.fwpkg.\n\n\t"
            "Skip extra checks before adding taskqueue. (Useful when adding "
            "many flashfwpkg taskqueue items in sequence.)\n\texample: flashfwpkg "
            "component.fwpkg --ignorechecks"
            "\n\n\t Uploading component "
            "flashfwpkg create taskqueue using target and flashes.\n\texample: flashfwpkg "
            "component.fwpkg --targets <id>",
            "summary": "Flashes fwpkg components using the iLO repository.",
            "aliases": ["fwpkg"],
            "auxcommands": [
                "UploadComponentCommand",
                "UpdateTaskQueueCommand",
                "FirmwareUpdateCommand",
                "FwpkgCommand",
            ],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main fwpkg worker function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            if "-" in line[0] and "thirdparty" in line[0]:
                line.insert(0, "dummy_arg")
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.fwpkgvalidation(options)

        if self.rdmc.app.typepath.defs.isgen9:
            LOGGER.error("iLO Repository commands are only available on iLO 5.")
            raise IncompatibleiLOVersionError("iLO Repository commands are only available on iLO 5.")

        if self.rdmc.app.getiloversion() <= 5.120 and options.fwpkg.lower().startswith("iegen10"):
            raise IncompatibleiLOVersionError(
                "Please upgrade to iLO 5 1.20 or greater to ensure correct flash of this firmware."
            )

        updateservice_url = "/redfish/v1/UpdateService/"
        # Get handler to check Accept3rdPartyFirmware enabled or disabled
        get_content = self.rdmc.app.get_handler(updateservice_url, silent=True, service=True).dict
        accept3rdpartyfw = get_content["Oem"]["Hpe"]["Accept3rdPartyFirmware"]
        line_l = line[0].lower()
        if (
            line
            and ".fwpkg" not in line_l
            and (".pup" in line_l or ".hpb" in line_l or ".fup" in line_l or ".bin" in line_l)
        ):
            if not accept3rdpartyfw and not options.enable_thirdparty_fw:
                raise InvalidCommandLineErrorOPTS(
                    "Please use the option --enable_thirdparty_fw to flash Third Party Firmware Update Packages\n"
                )

        if options.enable_thirdparty_fw:
            self.rdmc.ui.printer("Enabling Third party Firmware flashing.\n")
            self.rdmc.app.patch_handler(
                updateservice_url,
                {"Oem": {"Hpe": {"Accept3rdPartyFirmware": True}}},
                silent=True,
                service=True,
            )
            if "dummy" in line[0]:
                return ReturnCodes.SUCCESS
        elif options.disable_thirdparty_fw:
            self.rdmc.ui.printer("Disabling Third party Firmware flashing.\n")
            self.rdmc.app.patch_handler(
                updateservice_url,
                {"Oem": {"Hpe": {"Accept3rdPartyFirmware": False}}},
                silent=True,
                service=True,
            )
            if "dummy" in line[0]:
                return ReturnCodes.SUCCESS

        tempdir = ""
        fwpkg_l = options.fwpkg.lower()
        if (
            not fwpkg_l.endswith(".fwpkg")
            and not fwpkg_l.endswith(".fup")
            and not fwpkg_l.endswith(".hpb")
            and not fwpkg_l.endswith(".bin")
            and not fwpkg_l.endswith(".pup")
        ):
            LOGGER.error("Invalid file type. Please make sure the file provided is a valid .fwpkg file type.")
            raise InvalidFileInputError(
                "Invalid file type. Please make sure the file provided is a valid .fwpkg file type."
            )

        try:
            components, tempdir, comptype, _ = self.preparefwpkg(self, options.fwpkg, options)
            if comptype == "D":
                LOGGER.error("Component Type D, Unable to flash this fwpkg file.")
                raise InvalidFileInputError("Unable to flash this fwpkg file.")

            elif options.targets:
                for component in components:
                    component_l = component.lower()
                    if (
                        component_l.endswith(".fwpkg")
                        or component_l.endswith(".hpb")
                        or component_l.endswith(".HPb")
                        or component_l.endswith(".bin")
                        or component_l.endswith(".fup")
                        or component_l.endswith(".pup")
                        or component_l.endswith(".zip")
                    ):
                        uploadcommand = "--component %s" % component
                    else:
                        uploadcommand = "--component %s" % os.path.join(tempdir, component)
                    if options.forceupload:
                        uploadcommand += " --forceupload"
                    self.rdmc.ui.printer("Uploading firmware: %s\n" % os.path.basename(component))
                    self.auxcommands["uploadcomp"].run(uploadcommand)
                    targets = []
                    target_list = options.targets.split(",")
                    for target in target_list:
                        target_url = "/redfish/v1/UpdateService/FirmwareInventory/" + str(target) + "/"
                        targets.append(target_url)
                    verified = self.verify_targets(options.targets)
                    if not verified:
                        self.rdmc.ui.error("Provided target was not available, Please provide valid target id\n")
                        return ReturnCodes.INVALID_TARGET_ERROR
                    path = "/redfish/v1/UpdateService/UpdateTaskQueue/"
                    try:
                        self.taskqueuecheck()
                    except TaskQueueError as excp:
                        if options.ignore:
                            self.rdmc.ui.warn(str(excp) + "\n")
                        else:
                            raise excp

                    comp = "%s" % os.path.basename(component)
                    try:
                        newtask = {
                            "Name": "Update-%s" % (str(randint(0, 1000000)),),
                            "Command": "ApplyUpdate",
                            "Filename": os.path.basename(comp),
                            "UpdatableBy": ["Bmc"],
                            "TPMOverride": options.tover,
                            "Targets": targets,
                        }
                        self.rdmc.ui.printer('payload: "%s"\n' % newtask)
                    except ValueError:
                        pass
                    self.rdmc.ui.printer('Creating task: "%s"\n' % newtask["Name"])
                    self.rdmc.app.post_handler(path, newtask)
                    self.cmdbase.logout_routine(self, options)
                    # Return code
                    return ReturnCodes.SUCCESS

            elif comptype in ["C", "BC"]:
                try:
                    self.taskqueuecheck()
                except TaskQueueError as excp:
                    if options.ignore:
                        self.rdmc.ui.warn(str(excp) + "\n")
                    else:
                        raise excp
            self.applyfwpkg(options, tempdir, components, comptype)

            if comptype == "A":
                message = "Firmware has successfully been flashed.\n"
                if "ilo" in options.fwpkg.lower():
                    message += "iLO will reboot to complete flashing. Session will be" " terminated.\n"
            elif comptype in ["B", "BC"]:
                message = (
                    "Firmware has successfully been flashed and a reboot is required for "
                    "this firmware to take effect.\n"
                )
            elif comptype in ["C", "BC"]:
                message = "This firmware is set to flash on reboot.\n"
            if ".bin" not in components[0]:
                if "blobstore" not in self.rdmc.app.redfishinst.base_url:
                    if not self.auxcommands["uploadcomp"].wait_for_state_change():
                        # Failed to upload the component.
                        raise FirmwareUpdateError("Error while processing the component.")

            self.rdmc.ui.printer(message)

        except (FirmwareUpdateError, UploadError) as excp:
            raise excp
        except UploadPowerOffError:
            self.rdmc.ui.error("Server power is On,Kindly Power Off the server,to be able to flash the B- component \n")
            return ReturnCodes.UPLOAD_POWEROFF_ERROR

        finally:
            if tempdir:
                shutil.rmtree(tempdir)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def taskqueuecheck(self):
        """Check taskqueue for potential issues before starting"""

        select = "ComputerSystem."
        results = self.rdmc.app.select(selector=select, path_refresh=True)

        try:
            results = results[0]
        except:
            pass

        powerstate = results.resp.dict["PowerState"]
        tasks = self.rdmc.app.getcollectionmembers("/redfish/v1/UpdateService/UpdateTaskQueue/")

        for task in tasks:
            if task["State"] == "Exception":
                raise TaskQueueError(
                    "Exception found in taskqueue which will "
                    "prevent firmware from flashing. Please run "
                    "iLOrest command: taskqueue --cleanqueue to clear"
                    " any errors before continuing."
                )
            if task["UpdatableBy"] == "Uefi" and not powerstate == "Off" or task["Command"] == "Wait":
                raise TaskQueueError(
                    "Taskqueue item found that will "
                    "prevent firmware from flashing immediately. Please "
                    "run iLOrest command: taskqueue --resetqueue to "
                    "reset the queue if you wish to flash immediately "
                    "or include --ignorechecks to add this firmware "
                    "into the task queue anyway."
                )
        if tasks:
            self.rdmc.ui.warn(
                "Items are in the taskqueue that may delay the flash until they "
                "are finished processing. Use the taskqueue command to monitor updates.\n"
            )

    def is_ripcord(self):
        chassis_path = "/redfish/v1/Chassis/1/"
        chassis_info = self.rdmc.app.get_handler(chassis_path, silent=True)
        if (
            chassis_info.obj["SKU"] == "RIPCORD"
            or "Gen12" in chassis_info.obj["Model"]
            or "DL340" in chassis_info.obj["Model"]
            or "DL320" in chassis_info.obj["Model"]
        ):
            return True
        else:
            return False

    def get_comp_type(self, payload, cmdname, targetupdate):
        """Get's the component type and returns it

        :param payload: json payload of .fwpkg file
        :type payload: dict.
        :returns: returns the type of component. Either A,B,C, or D.
        :rtype: string
        """
        ctype = ""
        if "Uefi" in payload["UpdatableBy"] and "RuntimeAgent" in payload["UpdatableBy"]:
            ctype = "D"
        elif "Uefi" in payload["UpdatableBy"] and "Bmc" in payload["UpdatableBy"]:
            fw_url = "/redfish/v1/UpdateService/FirmwareInventory/" + "?$expand=."
            data = self.rdmc.app.get_handler(fw_url, silent=True).dict["Members"]
            da_flag = False
            cc_flag = False
            if data is not None:
                type_set = None
                for fw in data:
                    for device in payload["Devices"]["Device"]:
                        if fw["Oem"]["Hpe"].get("Targets") is not None:
                            if device["Target"] in fw["Oem"]["Hpe"]["Targets"]:
                                # print devicecontext in debugger
                                LOGGER.info("DeviceContext", fw["Oem"]["Hpe"]["DeviceContext"])
                                if (
                                    "Slot=" in fw["Oem"]["Hpe"]["DeviceContext"]
                                    and ":" in fw["Oem"]["Hpe"]["DeviceContext"]
                                ):
                                    cc_flag = True
                                else:
                                    da_flag = True
                if cc_flag and da_flag:
                    ctype = "BC"
                    type_set = True
                elif cc_flag and not da_flag:
                    ctype = "B"
                    type_set = True
                elif not cc_flag and da_flag:
                    ctype = "C"
                    type_set = True
                if type_set is None:
                    LOGGER.error("Component type is not identified, Please check if the particular H/W is present")
                    ilo_ver_int = self.rdmc.app.getiloversion()
                    ilo_ver = str(ilo_ver_int)
                    error_msg = (
                        "Cannot flash the component on this server, check whether the component is fwpkg-v2 "
                        "or check whether the server is iLO"
                    )
                    if ilo_ver_int >= 6:
                        error_msg = (
                            error_msg + ilo_ver[0] + ", FW is above 1.50 or the particular drive HW is present\n"
                        )
                    else:
                        error_msg = (
                            error_msg + ilo_ver[0] + ", FW is above 2.30 or the particular drive HW is present\n"
                        )
                    raise IncompatibleiLOVersionError(error_msg)
        else:
            b_minus_flag = False
            for device in payload["Devices"]["Device"]:
                for image in device["FirmwareImages"]:
                    if "DirectFlashOk" not in list(image.keys()):
                        raise InvalidFileInputError("Cannot flash this firmware.")
                    if "ServerPowerOff" in image and image["ServerPowerOff"]:
                        b_minus_flag = True
                    if not b_minus_flag:
                        if image["DirectFlashOk"]:
                            ctype = "A"
                            if image["ResetRequired"]:
                                ctype = "B"
                                break
                        elif image["UefiFlashable"]:
                            ctype = "C"
                            break
                        else:
                            ctype = "D"
                    elif b_minus_flag and cmdname == "uploadcomp" and not targetupdate:
                        if image["DirectFlashOk"]:
                            ctype = "A"
                            if image["ResetRequired"]:
                                ctype = "B"
                                break
                        elif image["UefiFlashable"]:
                            ctype = "C"
                            break
                        else:
                            ctype = "D"

                    else:
                        systems_path = "/redfish/v1/Systems/1/"
                        systems_info = self.rdmc.app.get_handler(systems_path, silent=True)
                        if systems_info.obj["PowerState"] == "On" and not (
                            self.is_ripcord() and "U69" in image["FileName"]
                        ):
                            raise UploadPowerOffError(
                                "Server power is On, Kindly power off the server to be able to be flash the B- component on the server \n"
                            )

                        else:
                            if self.is_ripcord() and "U69" in image["FileName"]:
                                LOGGER.info(
                                    "Flashing the B- Bios component , Please perform cold reboot of the server for iLO to complete the flashing ."
                                )
                            ctype = "B"
                    b_minus_flag = False
        LOGGER.info("Component Type identified is {}".format(ctype))
        return ctype

    @staticmethod
    def preparefwpkg(self, pkgfile, options):
        """Prepare fwpkg file for flashing

        :param pkgfile: Location of the .fwpkg file
        :type pkgfile: string.
        :returns: returns the files needed to flash, directory they are located
                                                            in, and type of file.
        :rtype: string, string, string
        """
        files = []
        imagefiles = []
        payloaddata = None
        tempdir = tempfile.mkdtemp()
        pldmflag = False
        pkgfile_l = pkgfile.lower()
        target = False
        if "update_target" in options:
            if options.update_target:
                target = True
            else:
                target = False

        if (
            not pkgfile_l.endswith(".fup")
            and not pkgfile_l.endswith(".hpb")
            and not pkgfile_l.endswith(".bin")
            and not pkgfile_l.endswith(".pup")
        ):
            try:
                zfile = zipfile.ZipFile(pkgfile)
                zfile.extractall(tempdir)
                zfile.close()
            except Exception as excp:
                raise InvalidFileInputError("Unable to unpack file. " + str(excp))

            files = os.listdir(tempdir)

            if "payload.json" in files:
                with open(os.path.join(tempdir, "payload.json"), encoding="utf-8") as pfile:
                    data = pfile.read()
                payloaddata = json.loads(data)
            else:
                raise InvalidFileInputError("Unable to find payload.json in fwpkg file.")

        if (
            not pkgfile_l.endswith(".fup")
            and not pkgfile_l.endswith(".hpb")
            and not pkgfile_l.endswith(".bin")
            and not pkgfile_l.endswith(".pup")
        ):
            comptype = self.auxcommands["flashfwpkg"].get_comp_type(payloaddata, self.cmdbase.name, target)
        else:
            comptype = "A"

        results = self.rdmc.app.getprops(selector="UpdateService.", props=["Oem/Hpe/Capabilities"])
        if comptype in ["C", "BC"]:
            imagefiles = [self.auxcommands["flashfwpkg"].type_c_change(tempdir, pkgfile)]
        else:
            if (
                not pkgfile_l.endswith("fup")
                and not pkgfile_l.endswith(".hpb")
                and not pkgfile_l.endswith(".bin")
                and not pkgfile_l.endswith(".pup")
            ):
                for device in payloaddata["Devices"]["Device"]:
                    for firmwareimage in device["FirmwareImages"]:
                        if "PLDMImage" in firmwareimage and firmwareimage["PLDMImage"]:
                            pldmflag = True
                        if firmwareimage["FileName"] not in imagefiles:
                            imagefiles.append(firmwareimage["FileName"])

        if (
            "blobstore" in self.rdmc.app.redfishinst.base_url
            and comptype in ["A", "B", "BC"]
            and results
            and "UpdateFWPKG" in results[0]["Oem"]["Hpe"]["Capabilities"]
        ):
            dll = BlobStore2.gethprestchifhandle()
            dll.isFwpkg20.argtypes = [c_char_p, c_int]
            dll.isFwpkg20.restype = c_bool

            with open(pkgfile, "rb") as fwpkgfile:
                fwpkgdata = fwpkgfile.read()

            fwpkg_buffer = ctypes.create_string_buffer(fwpkgdata)
            if dll.isFwpkg20(fwpkg_buffer, 2048):
                imagefiles = [pkgfile]
                tempdir = ""
        if (
            pkgfile_l.endswith(".hpb")
            or pkgfile_l.endswith(".fup")
            or pkgfile_l.endswith(".pup")
            or pkgfile_l.endswith(".bin")
        ):
            imagefiles = [pkgfile]

        elif self.rdmc.app.getiloversion() > 5.230 and payloaddata.get("PackageFormat") == "FWPKG-v2":
            imagefiles = [pkgfile]
        return imagefiles, tempdir, comptype, pldmflag

    def type_c_change(self, tdir, pkgloc):
        """Special changes for type C

        :param tempdir: path to temp directory
        :type tempdir: string.
        :param components: components to upload
        :type components: list.

        :returns: The location of the type C file to upload
        :rtype: string.
        """

        shutil.copy(pkgloc, tdir)

        fwpkgfile = os.path.split(pkgloc)[1]
        zfile = fwpkgfile[:-6] + ".zip"
        zipfileloc = os.path.join(tdir, zfile)

        os.rename(os.path.join(tdir, fwpkgfile), zipfileloc)

        return zipfileloc

    def applyfwpkg(self, options, tempdir, components, comptype):
        """Apply the component to iLO

        :param options: command line options
        :type options: list.
        :param tempdir: path to temp directory
        :type tempdir: string.
        :param components: components to upload
        :type components: list.
        :param comptype: type of component. Either A,B,C, or D.
        :type comptype: str.
        """

        for component in components:
            component_l = component.lower()
            if (
                component_l.endswith(".fwpkg")
                or component_l.endswith(".hpb")
                or component_l.endswith(".bin")
                or component_l.endswith(".fup")
                or component_l.endswith(".pup")
                or component_l.endswith(".zip")
            ):
                uploadcommand = "--component %s" % component
            else:
                uploadcommand = "--component %s" % os.path.join(tempdir, component)

            if options.forceupload:
                uploadcommand += " --forceupload"
            if comptype in ["A", "B"]:
                LOGGER.info("Setting --update_target --update_repository options as it is A or B")
                uploadcommand += " --update_target --update_repository"
            elif comptype in ["BC"]:
                LOGGER.info("Setting --update_target --update_repository options and " "calling uploadcomp once.")
                uploadcommand1 = uploadcommand + " --update_repository --update_target"
                if options.tover:
                    LOGGER.info("Setting --tpmover if tpm enabled.")
                    uploadcommand1 += " --tpmover"
                ret = self.auxcommands["uploadcomp"].run(uploadcommand1)
                if ret != ReturnCodes.SUCCESS:
                    raise UploadError("Error uploading component.")
                LOGGER.info(
                    "Continuing the flow with --component such that uploadcomp"
                    " is called again to upload component to the repository."
                )
            if options.update_srs:
                LOGGER.info("Setting --update_srs to store as recovery set.")
                uploadcommand += " --update_srs"
            if options.tover:
                LOGGER.info("Setting --tpmover if tpm enabled.")
                uploadcommand += " --tpmover"
            if "update_repository" not in uploadcommand:
                self.rdmc.ui.printer("Uploading firmware: %s\n" % os.path.basename(component))
            if "update_target" in uploadcommand:
                self.rdmc.ui.printer("Flashing firmware: %s\n" % os.path.basename(component))
            try:
                ret = self.auxcommands["uploadcomp"].run(uploadcommand)
                if ret != ReturnCodes.SUCCESS:
                    raise UploadError
            except UploadError:
                if comptype in ["A", "B", "BC"]:
                    select = self.rdmc.app.typepath.defs.hpilofirmwareupdatetype
                    results = self.rdmc.app.select(selector=select)

                    try:
                        results = results[0]
                    except:
                        pass

                    if results:
                        update_path = results.resp.request.path
                        error = self.rdmc.app.get_handler(update_path, silent=True)
                        self.auxcommands["firmwareupdate"].printerrmsg(error)
                    else:
                        raise FirmwareUpdateError("Error occurred while updating the firmware.")
                else:
                    raise UploadError("Error uploading component.")

            if comptype in ["C", "BC"]:
                self.rdmc.ui.warn("Setting a taskqueue item to flash UEFI flashable firmware.\n")
                path = "/redfish/v1/updateservice/updatetaskqueue"
                newtask = {
                    "Name": "Update-%s" % (str(randint(0, 1000000)),),
                    "Command": "ApplyUpdate",
                    "Filename": os.path.basename(component),
                    "UpdatableBy": ["Uefi"],
                    "TPMOverride": options.tover,
                }
                res = self.rdmc.app.post_handler(path, newtask)

                if res.status != 201:
                    raise TaskQueueError("Not able create UEFI task.\n")
                else:
                    self.rdmc.ui.printer(
                        "Created UEFI Task for Component " + os.path.basename(component) + " successfully.\n"
                    )

    def verify_targets(self, target):
        target_list = target.split(",")
        for target in target_list:
            try:
                target_url = "/redfish/v1/UpdateService/FirmwareInventory/" + target
                dd = self.rdmc.app.get_handler(target_url, service=True, silent=True)
                if dd.status == 404:
                    return False
            except:
                return False
        return True

    def fwpkgvalidation(self, options):
        """fwpkg validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument("fwpkg", help="""fwpkg file path""", metavar="[FWPKG]", default=None)
        customparser.add_argument(
            "--forceupload",
            dest="forceupload",
            action="store_true",
            help="Add this flag to force upload firmware with the same name already on the repository.",
            default=False,
        )
        customparser.add_argument(
            "--ignorechecks",
            dest="ignore",
            action="store_true",
            help="Add this flag to ignore all checks to the taskqueue before attempting to process the .fwpkg file.",
            default=False,
        )
        customparser.add_argument(
            "--enable_thirdparty_fw",
            dest="enable_thirdparty_fw",
            action="store_true",
            help="Add this flag to enable flashing third party firmware to server.",
            default=False,
        )
        customparser.add_argument(
            "--disable_thirdparty_fw",
            dest="disable_thirdparty_fw",
            action="store_true",
            help="Add this flag to disable flashing third party firmware to server.",
            default=False,
        )
        customparser.add_argument(
            "--tpmover",
            dest="tover",
            action="store_true",
            help="If set then the TPMOverrideFlag is passed in on the associated flash operations",
            default=False,
        )
        customparser.add_argument(
            "--update_srs",
            dest="update_srs",
            action="store_true",
            help="Add this flag to update the System Recovery Set with the uploaded firmware. "
            "NOTE: This requires an account login with the system recovery set privilege.",
            default=False,
        )
        customparser.add_argument(
            "--targets",
            help="If targets value specify a comma separated\t" "firmwareinventory id only",
            metavar="targets_indices",
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" RawHead Command for rdmc """

import json

import redfish

try:
    from rdmc_helper import InvalidCommandLineErrorOPTS, ReturnCodes
except ImportError:
    from ilorest.rdmc_helper import InvalidCommandLineErrorOPTS, ReturnCodes


class RawHeadCommand:
    """Raw form of the head command"""

    def __init__(self):
        self.ident = {
            "name": "rawhead",
            "usage": None,
            "description": "Run to to retrieve data from the "
            'passed in path\n\texample: rawhead "/redfish/v1/systems/'
            '(system ID)"\n',
            "summary": "Raw form of the HEAD command.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main raw head worker function

        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.headvalidation(options)

        if options.path.startswith('"') and options.path.endswith('"'):
            options.path = options.path[1:-1]

        results = self.rdmc.app.head_handler(options.path, silent=options.silent, service=options.service)

        content = None
        tempdict = dict()

        if results and results.status == 200:
            if results._http_response:
                content = results.getheaders()
            else:
                content = results._headers

            tempdict = dict(content)

            if options.filename:
                output = json.dumps(tempdict, indent=2, cls=redfish.ris.JSONEncoder, sort_keys=True)
                filehndl = open(options.filename[0], "w")
                filehndl.write(output)
                filehndl.close()

                self.rdmc.ui.printer("Results written out to '%s'.\n" % options.filename[0])
            else:
                if options.service:
                    self.rdmc.ui.printer("%s\n" % tempdict)
                else:
                    self.rdmc.ui.print_out_json(tempdict)
        else:
            return ReturnCodes.NO_CONTENTS_FOUND_FOR_OPERATION

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def headvalidation(self, options):
        """Raw head validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options, skipbuild=True)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "path",
            help="Uri on iLO to query HEAD.",
        )
        customparser.add_argument(
            "--silent",
            dest="silent",
            action="store_true",
            help="""Use this flag to silence responses""",
            default=None,
        )
        customparser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="""Use the provided filename to perform operations.""",
            action="append",
            default=None,
        )
        customparser.add_argument(
            "--service",
            dest="service",
            action="store_true",
            help="""Use this flag to enable service mode and increase the function speed""",
            default=False,
        )
